<?php

use Bitrix\Main\Application;
use Bitrix\Main\ArgumentException;
use Bitrix\Main\Config\Option;
use Bitrix\Main\Context;
use Bitrix\Main\DB\SqlQueryException;
use Bitrix\Main\EventManager;
use Bitrix\Main\IO\Directory;
use Bitrix\Main\IO\File;
use Bitrix\Main\Loader;
use Bitrix\Main\LoaderException;
use Bitrix\Main\Localization\Loc;
use Bitrix\Main\ModuleManager;
use Bitrix\Main\SystemException;
use Boxberry\Bitrix\OrdersTable;
use Boxberry\Bitrix\CitiesFullTable;
use Boxberry\Bitrix\PointsForParcelsTable;

Loc::loadMessages(__FILE__);

try {
    Loader::includeModule('sale');

    Loader::registerAutoLoadClasses('boxberry.delivery', [
        'OrdersTable' => 'lib/boxberry/bitrix/OrdersTable.php',
        'CitiesFullTable' => 'lib/boxberry/bitrix/CitiesFullTable.php',
        'PointsForParcelsTable' => 'lib/boxberry/bitrix/PointsForParcelsTable.php',
    ]);

    Loader::autoLoad('OrdersTable');
    Loader::autoLoad('CitiesFullTable');
    Loader::autoLoad('PointsForParcelsTable');

} catch (LoaderException $e) {
}

class boxberry_delivery extends CModule
{
    public $MODULE_NAME;
    public $MODULE_DESCRIPTION;
    public $MODULE_VERSION;
    public $MODULE_VERSION_DATE;
    public $MODULE_ID = 'boxberry.delivery';
    public $MODULE_SORT = 10000;
    public $SHOW_SUPER_ADMIN_GROUP_RIGHTS;
    public $MODULE_GROUP_RIGHTS = 'Y';
    public $PARTNER_NAME;
    public $PARTNER_URI;
    private string $NEED_MAIN_VERSION = '22.0.0';
    private array $NEED_MODULES = ['sale'];
    private string $NEED_PHP_VERSION = '8.1.0';
    private string $sourceDir;
    private ?string $docRoot;
    private ?string $handlerPath;

    public function __construct()
    {

        $this->docRoot = Application::getDocumentRoot();
        $this->sourceDir = __DIR__;
        $this->handlerPath = $this->docRoot . Option::get('sale', 'delivery_handles_custom_path', '/bitrix/php_interface/include/sale_delivery/');

        $arModuleVersion = [];
        include(dirname(__FILE__) . '/version.php');
        $this->MODULE_VERSION = $arModuleVersion['VERSION'];
        $this->MODULE_VERSION_DATE = $arModuleVersion['VERSION_DATE'];
        $this->MODULE_NAME = Loc::getMessage('BOXBERRY_MODULE_NAME');
        $this->MODULE_DESCRIPTION = Loc::getMessage('BOXBERRY_MODULE_DESCRIPTION');
        $this->PARTNER_NAME = Loc::getMessage('PARTNER_NAME');
        $this->PARTNER_URI = Loc::getMessage('PARTNER_URI');
    }

    /**
     * Метод устанавливает модуль в систему и создает таблицы в БД
     * @throws SystemException
     * @throws ArgumentException
     */
    public function DoInstall(): void
    {
        try {
            $this->checkModulesVersion();
        } catch (SystemException $e) {
            $this->showErrorForm($e->getMessage());
        }

        $this->InstallFiles();
        $this->InstallDB();
        $this->InstallEvents();
        RegisterModule($this->MODULE_ID);
        $this->installAgents();
    }

    /**
     * Метод удаляет модуль из системы и удаляет таблицы из БД если пользователь выбрал эту опцию
     * @throws SqlQueryException
     */
    public function DoUninstall(): void
    {
        $context = Application::getInstance()->getContext();
        $request = $context->getRequest();

        if (!$request->getPost('uninstall') && bitrix_sessid_post()) {
            $this->showUninstallTablesForm();
        }

        if ($request->getPost('droptables') === 'Y') {
            $this->UnInstallDB();
        }

        $this->UnInstallFiles();
        $this->UnInstallEvents();

        CAgent::RemoveModuleAgents($this->MODULE_ID);

        UnRegisterModule($this->MODULE_ID);
    }

    public function installAgents(): void
    {
        CAgent::AddAgent('Boxberry\Bitrix\Agents\Tracking::syncStatuses();', $this->MODULE_ID, 'N', 3600);
        CAgent::AddAgent('Boxberry\Bitrix\Agents\KeysChecker::checkToken();', $this->MODULE_ID, 'N', 3600);
        CAgent::AddAgent('Boxberry\Bitrix\Agents\CitiesUpdate::run();', $this->MODULE_ID, 'N', 86400);
        CAgent::AddAgent('Boxberry\Bitrix\Agents\PointsForParcelsUpdate::run();', $this->MODULE_ID, 'N', 21600);
    }

    /**
     * Метод создает таблицы модуля в БД
     * @throws ArgumentException
     * @throws SystemException
     */
    public function InstallDB(): bool
    {
        $connection = Application::getConnection();

        if (!$connection->isTableExists(OrdersTable::getTableName())) {
            OrdersTable::getEntity()->createDbTable();
        }

        if (!$connection->isTableExists(PointsForParcelsTable::getTableName())) {
            PointsForParcelsTable::getEntity()->createDbTable();
        }

        if (!$connection->isTableExists(CitiesFullTable::getTableName())) {
            CitiesFullTable::getEntity()->createDbTable();

            $connection->createIndex(CitiesFullTable::getTableName(), 'idx_code', 'CODE');
            $connection->createIndex(CitiesFullTable::getTableName(), 'idx_name', 'NAME');
            $connection->createIndex(CitiesFullTable::getTableName(), 'idx_region_district', ['REGION', 'DISTRICT']);
        }

        return true;
    }

    /**
     * Метод удаляет таблицы модуля из БД
     * @throws SqlQueryException
     */
    public function UnInstallDB(): void
    {
        $connection = Application::getConnection();

        if ($connection->isTableExists(OrdersTable::getTableName())) {
            $connection->dropTable(OrdersTable::getTableName());
        }

        if ($connection->isTableExists(CitiesFullTable::getTableName())) {
            $connection->dropTable(CitiesFullTable::getTableName());
        }

        if ($connection->isTableExists(PointsForParcelsTable::getTableName())) {
            $connection->dropTable(PointsForParcelsTable::getTableName());
        }
    }

    public function InstallEvents()
    {
        $eventManager = EventManager::getInstance();
        $eventManager->registerEventHandler('main', 'OnAdminListDisplay', $this->MODULE_ID, '\Boxberry\Bitrix\Handlers\Order', 'addOnAdminListDisplayMenu');
        $eventManager->registerEventHandler('main', 'OnEpilog', $this->MODULE_ID, '\Boxberry\Bitrix\Handlers\Order', 'handleOnEpilog');
    }

    public function UnInstallEvents()
    {
        $eventManager = EventManager::getInstance();
        $eventManager->unRegisterEventHandler('main', 'OnAdminListDisplay', $this->MODULE_ID, '\Boxberry\Bitrix\Handlers\Order', 'addOnAdminListDisplayMenu');
        $eventManager->unRegisterEventHandler('main', 'OnEpilog', $this->MODULE_ID, '\Boxberry\Bitrix\Handlers\Order', 'handleOnEpilog');
    }

    /**
     * Метод копирует файлы модуля в систему
     * @return void
     */
    public function InstallFiles(): void
    {
        CopyDirFiles($this->sourceDir . '/js/', $this->docRoot . '/bitrix/js/' . $this->MODULE_ID, true, true);
        CopyDirFiles($this->sourceDir . '/images/', $this->docRoot . '/bitrix/images/' . $this->MODULE_ID, true, true);

        File::putFileContents($this->handlerPath . 'boxberry/handler.php', File::getFileContents($this->sourceDir . '/handler.php'));
        File::putFileContents($this->docRoot . '/bitrix/admin/boxberry_orders.php', File::getFileContents($this->sourceDir . '/boxberry_orders.php'));
    }

    /**
     * Метод удаляет файлы модуля из системы
     * @return void
     */
    public function UnInstallFiles(): void
    {
        if (Directory::isDirectoryExists($this->handlerPath . 'boxberry')) {
            Directory::deleteDirectory($this->handlerPath . 'boxberry');
        }

        if (Directory::isDirectoryExists($this->docRoot . '/bitrix/js/' . $this->MODULE_ID)) {
            Directory::deleteDirectory($this->docRoot . '/bitrix/js/' . $this->MODULE_ID);
        }

        if (Directory::isDirectoryExists($this->docRoot . '/bitrix/images/' . $this->MODULE_ID)) {
            Directory::deleteDirectory($this->docRoot . '/bitrix/images/' . $this->MODULE_ID);
        }

        if (File::isFileExists($this->docRoot . '/bitrix/admin/boxberry_orders.php')) {
            File::deleteFile($this->docRoot . '/bitrix/admin/boxberry_orders.php');
        }
    }

    /**
     * @throws SystemException
     */
    private function checkModulesVersion(): void
    {
        $arErrors = [];

        if (version_compare(SM_VERSION, $this->NEED_MAIN_VERSION) < 0) {
            $arErrors[] = Loc::getMessage('NEED_MAIN_VERSION_ERROR', ['#NEED#' => $this->NEED_MAIN_VERSION]);
        }

        if (version_compare(PHP_VERSION, $this->NEED_PHP_VERSION) < 0) {
            $arErrors[] = Loc::getMessage('NEED_PHP_VERSION_ERROR', ['#NEED#' => $this->NEED_PHP_VERSION]);
        }

        if (!empty($this->NEED_MODULES)) {
            foreach ($this->NEED_MODULES as $module) {
                if (!ModuleManager::isModuleInstalled($module)) {
                    $arErrors[] = Loc::getMessage('NEED_MODULES_ERROR', ['#MODULE#' => $module]);
                }
            }
        }

        if (!empty($arErrors)) {
            throw new SystemException(implode('<br>', $arErrors));
        }
    }

    private function showErrorForm($errorMessage): void
    {
        global $APPLICATION;
        $excludeKeys = ['i', 'GLOBALS', 'strTitle', 'filepath', 'errorMessage'];

        foreach ($GLOBALS as $key => $value) {
            if (!in_array($key, $excludeKeys)) {
                global ${$key};
            }
        }

        $APPLICATION->SetTitle(Loc::getMessage('MODULE_INSTALL_ERROR'));
        include($_SERVER['DOCUMENT_ROOT'] . '/bitrix/modules/main/include/prolog_admin_after.php');
        ?>
        <form action="<?= $APPLICATION->GetCurPage(); ?>" method="post">
            <?= bitrix_sessid_post(); ?>
            <input type="hidden" name="lang" value="<?= Context::getCurrent()->getLanguage(); ?>"/>
            <input type="hidden" name="id" value="<?= htmlspecialcharsbx($this->MODULE_ID); ?>">

            <div class="adm-detail-content-wrap">
                <div class="adm-detail-content">
                    <div class="adm-detail-title">
                        <?= htmlspecialchars($errorMessage); ?>
                    </div>
                </div>
            </div>

            <div class="adm-detail-content-btns">
                <input type="button" value="<?= Loc::getMessage('BACK_BUTTON'); ?>" onclick="window.history.back();"/>
            </div>
        </form>
        <?php
        include($_SERVER['DOCUMENT_ROOT'] . '/bitrix/modules/main/include/epilog_admin.php');
        die();
    }


    /**
     * Метод отображает форму с выбором удаления таблиц из БД
     * @return void
     */
    private function showUninstallTablesForm(): void
    {
        global $APPLICATION;
        $excludeKeys = ['i', 'GLOBALS', 'strTitle', 'filepath'];

        foreach ($GLOBALS as $key => $value) {
            if (!in_array($key, $excludeKeys)) {
                global ${$key};
            }
        }

        $APPLICATION->SetTitle(Loc::getMessage('MODULE_UNINSTALL') . Loc::getMessage('BOXBERRY_MODULE_NAME'));
        include($_SERVER['DOCUMENT_ROOT'] . '/bitrix/modules/main/include/prolog_admin_after.php');
        ?>
        <form action="<?= $APPLICATION->GetCurPage(); ?>" method="post">
            <?= bitrix_sessid_post(); ?>
            <input type="hidden" name="lang" value="<?= Context::getCurrent()->getLanguage(); ?>"/>
            <input type="hidden" name="id" value="<?= htmlspecialcharsbx($this->MODULE_ID); ?>">
            <input type="hidden" name="uninstall" value="Y">

            <div class="adm-detail-content-wrap">
                <div class="adm-detail-content">
                    <div class="adm-detail-title">
                        <?= Loc::getMessage('MODULE_DROP_TABLES_DESC'); ?>
                    </div>

                    <div class="adm-detail-content-item-block">
                        <div class="adm-detail-content-item">
                            <p>
                                <input type="checkbox" name="droptables" id="droptables">
                                <label for="droptables"><?= Loc::getMessage('MODULE_LEAVE_TABLES_CONFIRM_LABEL'); ?></label>
                            </p>
                        </div>
                    </div>
                </div>
            </div>

            <div class="adm-detail-content-btns">
                <input type="submit" name="inst" value="<?= Loc::getMessage('MODULE_DELETE_TABLES_BUTTON'); ?>"
                       class="adm-btn-green"/>
            </div>
        </form>

        <script>
            document.addEventListener('DOMContentLoaded', function () {
                const droptablesCheckbox = document.getElementById('droptables');

                droptablesCheckbox.addEventListener('change', function () {
                    if (droptablesCheckbox.checked) {
                        droptablesCheckbox.value = 'Y';
                    } else {
                        droptablesCheckbox.value = '';
                    }
                });
            });
        </script>

        <?php
        include($_SERVER['DOCUMENT_ROOT'] . '/bitrix/modules/main/include/epilog_admin.php');
        die();
    }


}