<?php
namespace Boxberry\Bitrix;

use Bitrix\Main\Application;
use Bitrix\Main\ArgumentException;
use Bitrix\Main\DB\SqlQueryException;
use Bitrix\Main\Localization\Loc;
use Bitrix\Main\ObjectPropertyException;
use Bitrix\Main\ORM\Data\DataManager;
use Bitrix\Main\ORM\Fields\IntegerField;
use Bitrix\Main\ORM\Fields\StringField;
use Bitrix\Main\ORM\Fields\Validators\LengthValidator;
use Bitrix\Main\ORM\Query\Filter\Condition;
use Bitrix\Main\ORM\Query\Filter\ConditionTree;
use Bitrix\Main\ORM\Query\Query;
use Bitrix\Main\ORM\Query\Result;
use Bitrix\Main\SystemException;
use Bitrix\Sale\Location\LocationTable;
use Boxberry\Api\CountriesSettings;
use Boxberry\Bitrix\Entity\City;
use Boxberry\Bitrix\Helpers\Logger;
use Boxberry\Bitrix\Helpers\Options;

Loc::loadMessages(__FILE__);

class CitiesFullTable extends DataManager
{
    public const RU_COUNTRY_CODE = '643';
    public const TJ_COUNTRY_CODE = '762';
    public const BE_COUNTRY_CODE = '112';
    private const CITY_PREFIX = 'г';
    private const VILLAGE = 'VILLAGE';
    private const CITY = 'CITY';
    private static bool $isAddingCities = false;

    /**
     * Returns DB table name for entity.
     *
     * @return string
     */
    public static function getTableName()
    {
        return 'b_boxberry_cities_full';
    }

    /**
     * Returns entity map definition.
     *
     * @return array
     */
    public static function getMap()
    {
        return [
            (new IntegerField('ID'))
                ->configurePrimary(true)
                ->configureAutocomplete(true)
                ->configureTitle(Loc::getMessage('CITIES_ENTITY_ID_FIELD')),

            (new StringField('CODE'))
                ->configureTitle(Loc::getMessage('CITIES_ENTITY_CODE_FIELD'))
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('NAME'))
                ->configureTitle(Loc::getMessage('CITIES_ENTITY_NAME_FIELD'))
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('RECEPTION'))
                ->configureTitle(Loc::getMessage('CITIES_ENTITY_RECEPTION_FIELD'))
                ->addValidator(new LengthValidator(null, 128)),

            (new StringField('PICKUP_POINT'))
                ->configureTitle(Loc::getMessage('CITIES_ENTITY_PICKUP_POINT_FIELD'))
                ->addValidator(new LengthValidator(null, 128)),

            (new StringField('FOREIGN_RECEPTION_RETURNS'))
                ->configureTitle(Loc::getMessage('CITIES_ENTITY_FOREIGN_RECEPTION_RETURNS_FIELD'))
                ->addValidator(new LengthValidator(null, 128)),

            (new StringField('TERMINAL'))
                ->configureTitle(Loc::getMessage('CITIES_ENTITY_TERMINAL_FIELD'))
                ->addValidator(new LengthValidator(null, 128)),

            (new StringField('COURIER_DELIVERY'))
                ->configureTitle(Loc::getMessage('CITIES_ENTITY_COURIER_DELIVERY_FIELD'))
                ->addValidator(new LengthValidator(null, 128)),

            (new StringField('KLADR'))
                ->configureTitle(Loc::getMessage('CITIES_ENTITY_KLADR_FIELD'))
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('REGION'))
                ->configureTitle(Loc::getMessage('CITIES_ENTITY_REGION_FIELD'))
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('COUNTRY_CODE'))
                ->configureTitle(Loc::getMessage('CITIES_ENTITY_COUNTRY_CODE_FIELD'))
                ->addValidator(new LengthValidator(null, 128)),

            (new StringField('UNIQ_NAME'))
                ->configureTitle(Loc::getMessage('CITIES_ENTITY_UNIQ_NAME_FIELD'))
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('DISTRICT'))
                ->configureTitle(Loc::getMessage('CITIES_ENTITY_DISTRICT_FIELD'))
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('PREFIX'))
                ->configureTitle(Loc::getMessage('CITIES_ENTITY_PREFIX_FIELD'))
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('COURIER_RECEPTION'))
                ->configureTitle(Loc::getMessage('CITIES_ENTITY_COURIER_RECEPTION_FIELD'))
                ->addValidator(new LengthValidator(null, 128)),

        ];
    }

    public static function isEmpty(): bool
    {
        return self::getCount() === 0;
    }

    public static function truncate(): void
    {
        $connection = Application::getConnection();
        $connection->truncateTable(self::getTableName());
    }

    /**
     * @throws ArgumentException
     * @throws SqlQueryException
     * @throws SystemException
     * @throws \Exception
     */
    public static function addCities(array $cities): void
    {
        self::$isAddingCities = true;

        if (!self::isEmpty()) {
            self::truncate();
        }

        try {
            $cities = array_map(function ($city) {
                return array_combine(
                    array_map('\Bitrix\Main\Text\StringHelper::camel2snake', array_keys($city)),
                    $city
                );
            }, $cities);

            $newCities = [];
            $regionReplaces = self::getBbRegionReplaces();
            $cityReplaces = self::getBbCityReplaces();

            foreach ($cities as $city) {
                if ($city['pickup_point'] === '0' && $city['courier_delivery'] === '0') {
                    continue;
                }

                $city['name'] = trim(str_replace(array_keys($cityReplaces), array_values($cityReplaces), $city['name']));
                $city['region'] = trim(str_replace(array_keys($regionReplaces), array_values($regionReplaces), $city['region']));

                if ($city['name'] === Loc::getMessage('SEVASTOPOL')) {
                    $city['region'] = Loc::getMessage('CRYM');
                }

                $newCities[] = $city;
            }

            if (!empty($newCities)) {
                $chunks = array_chunk($newCities, 500);
                foreach ($chunks as $chunk) {
                    $result = self::addMulti($chunk, true);
                    if (!$result->isSuccess()) {
                        throw new \Exception(Loc::getMessage('CAN_NOT_UPDATE_DATA_IN_TABLE') . self::getTableName());
                    }
                }
            }
        } catch (\Exception $e) {
            if (Options::isEventLogEnabled()) {
                Logger::add($e->getMessage(), __METHOD__ . '();', $e);
            }
            throw $e;
        } finally {
            self::$isAddingCities = false;
        }
    }

    public static function getCityByCode(string $code): ?array
    {
        return self::getRow([
            'filter' => [
                '=CODE' => $code
            ]
        ]);
    }

    private static function getBbRegionReplaces(): array
    {
        return [
            '/' => '',
            Loc::getMessage('YUGRA') => '',
            Loc::getMessage('KUZBASS') => '',
            Loc::getMessage('ALANIA_WITH_SPACES') => Loc::getMessage('ALANIA'),
            Loc::getMessage('HANTY_MANSIYSKIY_AVTONOMNIY_OKRUG_WITH_CAPITAL_A') => Loc::getMessage('HANTY_MANSIYSKIY_AVTONOMNIY_OKRUG'),
            Loc::getMessage('RESPUBLIKA_START_WITH_SPACE') => '',
            Loc::getMessage('YAMALO_NENETSKIY_AVTONOMNIY_OKRUG') => Loc::getMessage('YAMALO_NENETSKIY'),
            Loc::getMessage('NENETSKIY') => Loc::getMessage('NENETSKIY_AVTONOMNIY_OKRUG'),
            Loc::getMessage('EVREYSKAYA') => Loc::getMessage('EVREYSKAYA_AVTONOMNAYA'),
            Loc::getMessage('OBLAST') => '',
            Loc::getMessage('RESPUBLIKA_WITH_CAPITAL_R') => '',
        ];
    }

    private static function getBbCityReplaces(): array
    {
        return [
            '"' => '',
            Loc::getMessage('ALMA_ATA') => Loc::getMessage('ALMATY'),
        ];
    }

    public static function getRegionReplaceMappings(): array
    {
        return [
            '(' => '',
            ')' => '',
            Loc::getMessage('OBLAST') => '',
            Loc::getMessage('RESPUBLIKA_WITH_CAPITAL_R') => '',
            Loc::getMessage('RESPUBLIKA') => '',
            Loc::getMessage('KRAY') => '',
            Loc::getMessage('RAION') => '',
            Loc::getMessage('YUGRA_WITH_CAPITAL_Y') => '',
            Loc::getMessage('KUZBASS_WITH_CAPITAL_K') => '',
            Loc::getMessage('_ULUS') => '',
        ];
    }

    public static function getCountryDistrictMappings(): array
    {
        return [
            Loc::getMessage('SEVERO-ZAPAD') => Loc::getMessage('SANKT-PETERBURG'),
            Loc::getMessage('CENTR') => Loc::getMessage('MOSKVA'),
        ];
    }

    public static function getVillagePrefix($villageName): string
    {
        $prefixes = self::getVillagePrefixMapping();
        $prefix = '';

        foreach ($prefixes as $key => $value) {
            if (str_ends_with($villageName, $key)) {
                $prefix = $value;
                break;
            }
        }

        return $prefix;
    }

    private static function getVillagePrefixMapping(): array
    {
        return [
            Loc::getMessage('_POSOYOLOK_GORODSKOGO_TIPA') => Loc::getMessage('PGT'),
            Loc::getMessage('_POSELOK_GORODSKOGO_TIPA') => Loc::getMessage('PGT'),
            Loc::getMessage('_RABOCHIY_POSYOLOK') => Loc::getMessage('RP'),
            Loc::getMessage('_RABOCHIY_POSELOK') => Loc::getMessage('RP'),
            Loc::getMessage('_DACHNY_POSYOLOK') => Loc::getMessage('DP'),
            Loc::getMessage('_DACHNY_POSELOK') => Loc::getMessage('DP'),
            Loc::getMessage('_POSOYOLOK') => Loc::getMessage('P'),
            Loc::getMessage('_POSELOK') => Loc::getMessage('P'),
            Loc::getMessage('_DEREVNYA') => Loc::getMessage('D'),
            Loc::getMessage('_SELO') => Loc::getMessage('S'),
            Loc::getMessage('_HUTOR') => Loc::getMessage('H'),
            Loc::getMessage('_STANICA') => Loc::getMessage('ST_CA'),
            Loc::getMessage('_MICRORAION') => Loc::getMessage('MKR'),
            Loc::getMessage('_POCHTOVOE_OTDELENIE') => Loc::getMessage('P_O'),
            Loc::getMessage('_GORODOK') => Loc::getMessage('GORODOK'),
            Loc::getMessage('_MASSIV') => Loc::getMessage('MASSIV'),
            Loc::getMessage('_SELSKOE_POSELENIE') => Loc::getMessage('S_P'),
            Loc::getMessage('_AUL') => Loc::getMessage('AUL'),
            Loc::getMessage('_ZHELEZNODOROJNAYA_STANCIYA') => Loc::getMessage('ZHS'),
            Loc::getMessage('_NASELYONNIY_PUNKT') => Loc::getMessage('NP'),
            Loc::getMessage('_NASELENNIY_PUNKT') => Loc::getMessage('NP'),
            Loc::getMessage('_SNT') => Loc::getMessage('SNT'),
        ];
    }

    private static function getVillagePrefixMappingUnion(): array
    {
        return [
            Loc::getMessage('_AGROGORODOK') => '',
            Loc::getMessage('_GORODSKOI_POSYOLOK') => '',
        ];
    }

    /**
     * @throws ObjectPropertyException
     * @throws SystemException
     * @throws ArgumentException
     * @throws \Exception
     */
    public static function getBbCityByCode($code, $select = ['*']): City
    {
        if (self::$isAddingCities) {
            throw new \Exception(Loc::getMessage('CITIES_UPDATE_IN_PROGRESS'));
        }

        $result = self::query()
            ->setSelect($select)
            ->where('CODE', $code)
            ->setCacheTtl(3600)
            ->cacheJoins(true)
            ->setLimit(1)
            ->exec()
            ->fetch();

        if (!$result) {
            throw new \Exception(Loc::getMessage('CITY_WITH_CODE_NOT_FOUND', ['#CODE#' => $code, '#TABLENAME#' => self::getTableName()]));
        }

        return new City($result);
    }

    /**
     * @throws ArgumentException
     * @throws SqlQueryException
     * @throws ObjectPropertyException
     * @throws SystemException
     * @throws \Exception
     */
    public static function getBbCityByBitrixLocationCode($locationCode): City
    {
        if (self::$isAddingCities) {
            throw new \Exception(Loc::getMessage('CITIES_UPDATE_IN_PROGRESS'));
        }

        if (!$result = self::getLocationArrayByCode($locationCode)) {
            throw new \Exception(Loc::getMessage('CITY_WITH_CODE_NOT_FOUND', ['#CODE#' => $locationCode, '#TABLENAME#' => LocationTable::getTableName()]));
        }

        $city = self::query()
            ->addSelect('*');

        $regionReplaceMappings = self::getRegionReplaceMappings();

        if (isset($result['COUNTRY']) && array_key_exists($result['COUNTRY'], CountriesSettings::getCountries())) {
            $countryCode = CountriesSettings::getCodeByName($result['COUNTRY']);
            $city->where('COUNTRY_CODE', $countryCode);
        } else {
            $countryCode = '';
        }

        if (isset($result['VILLAGE'])) {
            self::setVillage($result, $city, $countryCode, $locationCode);
        } elseif (isset($result['CITY'])) {
            self::setCity($result, $city, $countryCode);
        } else {
            throw new \Exception(Loc::getMessage('NO_MATCH_FOUND_FOR_THE_CITY_WITH_THE_CODE', ['#CODE#' => $locationCode, '#TABLENAME#' => self::getTableName()]));
        }

        $countryDistrictMappings = self::getCountryDistrictMappings();
        $countryDistrict = '';
        if (isset($result['COUNTRY_DISTRICT']) && in_array($result['COUNTRY_DISTRICT'], array_keys($countryDistrictMappings))) {
            $countryDistrict = trim(str_replace(array_keys($countryDistrictMappings), array_values($countryDistrictMappings), $result['COUNTRY_DISTRICT']));
        }

        if (isset($result['REGION'])) {
            self::setRegion($result, $city, $regionReplaceMappings, $countryCode, $countryDistrict);
        } elseif ($countryCode === self::RU_COUNTRY_CODE && $countryDistrict) {
            $city->where('REGION', $countryDistrict);
        }

        if (isset($result['SUBREGION'])) {
            self::setSubRegion($result, $city, $regionReplaceMappings, $countryCode);
        }

        $city
            ->setLimit(1)
            ->setCacheTtl(3600)
            ->cacheJoins(true);

        if (!$city = $city->exec()->fetch()) {
            throw new \Exception(Loc::getMessage('NO_MATCH_FOUND_FOR_THE_CITY_WITH_THE_CODE', ['#CODE#' => $locationCode, '#TABLENAME#' => self::getTableName()]));
        }

        return new City($city);
    }

    /**
     * @throws \Exception
     */
    private static function setVillage(array $result, Query $city, string $countryCode, string $locationCode): void
    {
        $villagePrefixMapping = self::getVillagePrefixMapping();
        $villagePrefixMappingUnion = self::getVillagePrefixMappingUnion();

        if ($countryCode === self::RU_COUNTRY_CODE) {
            $locationName = trim(str_replace(array_keys($villagePrefixMapping), '', $result['VILLAGE']));
            if ($prefix = self::getVillagePrefix($result['VILLAGE'])) {
                $city->where('PREFIX', $prefix);
            } else {
                $city->whereNot('PREFIX', self::CITY_PREFIX);
            }
        } else {
            $villagePrefixMapping = $villagePrefixMappingUnion + $villagePrefixMapping;
            $locationName = trim(str_replace(array_keys($villagePrefixMapping), '', $result['VILLAGE']));
            $city->where(Query::filter()
                ->logic(ConditionTree::LOGIC_OR)
                ->where('PREFIX', '')
                ->whereNot('PREFIX', self::CITY_PREFIX)
            );
        }

        $city->where('NAME', $locationName);
    }

    private static function setCity(array $result, Query $city, $countryCode): void
    {
        if ($countryCode === self::RU_COUNTRY_CODE) {
            $city->where('PREFIX', self::CITY_PREFIX);
        } else {
            $city->whereIn('PREFIX', ['', self::CITY_PREFIX]);
        }

        $city->where('NAME', $result['CITY']);
    }

    private static function setSubRegion(array $result, Query $city, $regionReplaceMappings, $countryCode): void
    {
        if ($countryCode === self::RU_COUNTRY_CODE) {
            $subregion = trim(str_replace(array_keys($regionReplaceMappings), '', $result['SUBREGION']));
            $city->whereIn('DISTRICT', [$subregion, '']);
        }
    }

    private static function setRegion(array $result, Query $city, array $regionReplaceMappings, $countryCode, string $countryDistrict): void
    {
        $region = trim(str_replace(array_keys($regionReplaceMappings), array_values($regionReplaceMappings), $result['REGION']));

        if ($countryCode === self::RU_COUNTRY_CODE) {

            $cityName = '';
            foreach ($city->getFilterHandler()->getConditions() as $condition) {
                if ($condition instanceof Condition && $condition->getColumn() === 'NAME') {
                    $cityName = $condition->getValue();
                    break;
                }
            }

            $conditions = [['REGION', $region]];

            if (!empty($cityName)) {
                $conditions[] = ['REGION', $cityName];
            }

            if (!empty($countryDistrict)) {
                $conditions[] = ['REGION', $countryDistrict];
            }

            $city
                ->where(Query::filter()
                    ->logic(ConditionTree::LOGIC_OR)
                    ->where($conditions));
        }

        if ($countryCode === self::BE_COUNTRY_CODE) {
            $city->where('REGION', $region);
        }
    }

    private static function getLocationArrayByCode($locationCode): array
    {
        $path = [];

        try {
            $res = self::getFullNode($locationCode)->fetchAll();

            foreach ($res as $item) {
                $path[$item['TYPE_CODE']] = $item['LNAME'];
            }

            return $path;
        } catch (\Exception $e) {
            if (Options::isEventLogEnabled()) {
                Logger::add($e->getMessage(), __METHOD__ . '();', $e);
            }

            return $path;
        }
    }

    /**
     * @throws \Exception
     */
    private static function getFullNode($locationCode): Result
    {
        $node = LocationTable::query()
            ->setSelect(['ID', 'LEFT_MARGIN', 'RIGHT_MARGIN', 'DEPTH_LEVEL'])
            ->where('CODE', $locationCode)
            ->setLimit(1)
            ->setCacheTtl(3600)
            ->cacheJoins(true)
            ->exec()
            ->fetch();

        if (!isset($node['ID'])) {
            throw new \Exception(Loc::getMessage('CITY_WITH_CODE_NOT_FOUND', ['#CODE#' => $locationCode, '#TABLENAME#' => LocationTable::getTableName()]));
        }

        return LocationTable::query()
            ->setSelect(['LNAME' => 'NAME.NAME', 'TYPE_CODE' => 'TYPE.CODE'])
            ->where([
                ['NAME.LANGUAGE_ID', 'ru'],
                ['LEFT_MARGIN', '<=', $node['LEFT_MARGIN']],
                ['RIGHT_MARGIN', '>=', $node['RIGHT_MARGIN']],
                ['DEPTH_LEVEL', '<=', $node['DEPTH_LEVEL']],
            ])
            ->setOrder(['LEFT_MARGIN' => 'asc'])
            ->setLimit($node['DEPTH_LEVEL'])
            ->setCacheTtl(3600)
            ->cacheJoins(true)
            ->exec();
    }

}