<?php

namespace Boxberry\Bitrix;

use Bitrix\Main\ArgumentException;
use Bitrix\Main\Engine\Response\AjaxJson;
use Bitrix\Main\Error;
use Bitrix\Main\ErrorCollection;
use Bitrix\Main\Localization\Loc;
use Bitrix\Main\ObjectPropertyException;
use Bitrix\Main\ORM\Data\DataManager;
use Bitrix\Main\ORM\Fields\IntegerField;
use Bitrix\Main\ORM\Fields\StringField;
use Bitrix\Main\ORM\Fields\Validators\LengthValidator;
use Bitrix\Main\Result;
use Bitrix\Main\Text\Encoding;

Loc::loadMessages(__FILE__);

class PointsForParcelsTable extends DataManager
{
    public static bool $addingPointsForParcels = false;

    /**
     * Returns DB table name for entity.
     *
     * @return string
     */
    public static function getTableName()
    {
        return 'b_boxberry_points_for_parcels';
    }

    /**
     * Returns entity map definition.
     *
     * @return array
     */
    public static function getMap()
    {
        return [
            (new IntegerField('ID'))
                ->configurePrimary(true)
                ->configureAutocomplete(true)
                ->configureTitle(Loc::getMessage('POINTS_FOR_PARCELS_ENTITY_ID_FIELD')),

            (new StringField('CODE'))
                ->configureTitle(Loc::getMessage('POINTS_FOR_PARCELS_ENTITY_CODE_FIELD'))
                ->addValidator(new LengthValidator(null, 128)),

            (new StringField('NAME'))
                ->configureTitle(Loc::getMessage('POINTS_FOR_PARCELS_ENTITY_NAME_FIELD'))
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('CITY'))
                ->configureTitle(Loc::getMessage('POINTS_FOR_PARCELS_ENTITY_CITY_FIELD'))
                ->addValidator(new LengthValidator(null, 255)),

        ];
    }

    /**
     * @throws \Exception
     */
    public static function addPointsForParcels(array $points): void
    {
        self::$addingPointsForParcels = true;

        if (empty($points)) {
            throw new \Exception(Loc::getMessage('EMPTY_POINTS_FOR_PARCELS_ARRAY'));
        }

        if (self::getCount() > 0) {
            self::truncate();
        }

        $result = self::addMulti($points, true);

        if (!$result->isSuccess()) {
            throw new \Exception(implode(', ', $result->getErrorMessages()));
        }

        self::$addingPointsForParcels = false;
    }

    public static function isEmpty(): bool
    {
        return self::getCount() === 0;
    }

    private static function truncate(): void
    {
        $connection = \Bitrix\Main\Application::getConnection();
        $connection->truncateTable(self::getTableName());
    }

    public static function getPointsForParcels($searchTerm): Result
    {
        $result = new Result();
        if (self::$addingPointsForParcels) {
            $result->addError(new Error(Loc::getMessage('POINTS_FOR_PARCELS_UPDATE_IN_PROGRESS')));
            return $result;
        }

        try {
            $query = self::query()
                ->addSelect('NAME')
                ->whereLike('NAME', '%' . Encoding::convertEncodingToCurrent($searchTerm) . '%')
                ->setLimit(10)
                ->exec()
                ->fetchAll();

            if (!$query) {
                $result->addError(new Error(Loc::getMessage('POINTS_NOT_FOUND')));
                return $result;
            }
        } catch (\Exception $e) {
            $result->addError(new Error($e->getMessage()));
            return $result;
        }

        $result->setData(array_column($query, 'NAME'));

        return $result;
    }

    public static function getPointCodeByName($name): string
    {
        try {
            $query = self::query()
                ->addSelect('CODE')
                ->where('NAME', Encoding::convertEncodingToCurrent($name))
                ->setLimit(1)
                ->setCacheTtl(3600)
                ->exec()
                ->fetch();
        } catch (\Exception $e) {
            return '';
        }

        return $query['CODE'] ?? '';
    }

    public static function getPointNameByCode($code): string
    {
        try {
            $query = self::query()
                ->addSelect('NAME')
                ->where('CODE', Encoding::convertEncodingToCurrent($code))
                ->setLimit(1)
                ->setCacheTtl(3600)
                ->exec()
                ->fetch();
        } catch (\Exception $e) {
            return '';
        }

        return $query['NAME'] ?? '';
    }


}