<?php

namespace Boxberry\Bitrix\Agents;

use Bitrix\Main\Config\Option;
use Bitrix\Sale\Internals\ShipmentTable;
use Boxberry\Api\Client;
use Boxberry\Api\Exceptions\BoxberryApiException;
use Boxberry\Bitrix\Handlers\Order;
use Boxberry\Bitrix\Helpers\Delivery;
use Boxberry\Bitrix\Helpers\Logger;
use Boxberry\Bitrix\Helpers\Options;
use Boxberry\Bitrix\OrdersTable;

class Tracking
{
    public const BB_SUCCESS_FINAL_STATUS_ID = 150;
    public const BB_RETURN_FINAL_STATUS_ID = 190;

    /**
     * Проверяет статусы заказов, если в настройках модуля включена синхронизация статусов и если настроены их соответствия.
     * Если статус заказа в службе доставки отличается от статуса в Битриксе, то статус заказа в Битриксе обновляется.
     *
     * 10.06.2024 - Добавлена синхронизация статусов отгрузок
     *
     * @return string
     */
    public static function syncStatuses(): string
    {
        $agentName = __METHOD__ . '();';

        $isEventLogEnabled = Options::isEventLogEnabled();
        $isSyncEnabled = Options::isStatusSyncEnabled();

        if (!$isSyncEnabled) {
            return $agentName;
        }

        $statusMappings = Options::getStatusMappings();
        $shipmentStatusMappings = Options::getShipmentStatusMappings();

        if (empty($statusMappings) && empty($shipmentStatusMappings)) {
            return $agentName;
        }

        $ordersLimit = (int)Options::getStatusSyncOrdersLimit();
        $excludeDays = (int)Options::getStatusSyncOrdersExcludeDays();

        $orders = OrdersTable::getOrdersForStatusSync($ordersLimit, $excludeDays);

        if (empty($orders)) {
            return $agentName;
        }

        $apiTokens = Options::getApiTokens();
        $deliveryIds = Delivery::getAllProfilesIds();

        $ordersByDeliveryService = [];
        foreach ($orders as $order) {
            $ordersByDeliveryService[$order['PARENT_DELIVERY_ID']][] = $order;
        }

        foreach ($ordersByDeliveryService as $Id => $orders) {
            $apiToken = $apiTokens[$Id];
            $client = new Client($apiToken, 30);

            $trackNumbers = array_column($orders, 'TRACK');

            try {
                $statuses = $client->getLastStatusData($trackNumbers);
            } catch (BoxberryApiException $e) {
                if ($isEventLogEnabled) {
                    Logger::Add($e->getMessage(), $agentName);
                }
                continue;
            }

            foreach ($orders as $order) {
                $trackNumber = $order['TRACK'];
                $statusId = $statuses->getLastStatusIdByTrack($trackNumber);
                $statusName = $statuses->getLastStatusNameByTrack($trackNumber);
                $statusDate = $statuses->getLastStatusDateByTrack($trackNumber);

                if ($statusId === '') {
                    continue;
                }

                self::handleShipmentStatusSync($order, $deliveryIds, $statusId, $isEventLogEnabled);

                $bitrixStatusId = Options::getBitrixStatusIdByBbStatusId($statusId);

                if (!$bitrixStatusId) {
                    continue;
                }

                if (!Options::isStatusExist($bitrixStatusId)) {
                    continue;
                }

                if ($order['BITRIX_STATUS_ID'] === $bitrixStatusId) {
                    continue;
                }

                $result = Order::changeBitrixOrderStatus($order['ORDER_ID'], $bitrixStatusId, $statusId);

                if (!$result->isSuccess()) {
                    continue;
                }

                try {
                    OrdersTable::update($order['ID'],
                        [
                            'BB_STATUS_ID' => $statusId,
                            'BB_STATUS_NAME' => $statusName,
                            'BB_STATUS_DATE' => $statusDate,
                            'BITRIX_STATUS_ID' => $bitrixStatusId
                        ]
                    );
                } catch (\Exception $e) {
                    if ($isEventLogEnabled) {
                        Logger::Add($e->getMessage(), $agentName);
                    }
                    continue;
                }

            }
        }

        return $agentName;

    }

    private static function handleShipmentStatusSync($order, $deliveryIds, $statusId, $isEventLogEnabled): void
    {
        if (!$shipmentStatusId = Options::getShipmentStatusIdByBbStatusId($statusId)) {
            return;
        }

        if (!Options::isStatusExist($shipmentStatusId)) {
            return;
        }

        try {
            $shipment = ShipmentTable::query()
                ->where('ORDER_ID', $order['ORDER_ID'])
                ->whereNot('SYSTEM', 'Y')
                ->whereIn('DELIVERY_ID', $deliveryIds)
                ->addSelect('ID')
                ->addSelect('ORDER_ID')
                ->addSelect('STATUS_ID')
                ->setLimit(1)
                ->exec()
                ->fetch();

            if (!$shipment) {
                return;
            }

            if ($shipment['STATUS_ID'] === $shipmentStatusId) {
                return;
            }

            $shipmentOrder = \Bitrix\Sale\Order::load($shipment['ORDER_ID'])->getShipmentCollection()->getItemById($shipment['ID']);

            if (!$shipmentOrder) {
                return;
            }

            $shipmentOrder->setField('STATUS_ID', $shipmentStatusId);

            if (Options::isNeedShippedFlag($shipmentStatusId)) {
                $shipmentOrder->setField('DEDUCTED', 'Y');
            }

            $shipmentOrder->save();
        } catch (\Exception $e) {
            if ($isEventLogEnabled) {
                Logger::Add($e->getMessage(), __METHOD__ . '();', $e);
            }
        }
    }
}