<?php

namespace Boxberry\Bitrix\Entity;

use Bitrix\Main\Error;
use Bitrix\Main\Event;
use Bitrix\Main\Result;
use Bitrix\Main\Web\Json;
use Bitrix\Sale\BasketItem;
use Bitrix\Sale\Order as BitrixOrder;
use Bitrix\Sale\Shipment;
use Bitrix\Sale\ShipmentCollection;
use Boxberry\Api\Client;
use Boxberry\Api\Entity\Customer;
use Boxberry\Api\Entity\Export;
use Boxberry\Api\Entity\Kurdost;
use Boxberry\Api\Entity\ParcelParams;
use Boxberry\Api\Entity\Shop;
use Boxberry\Bitrix\CitiesFullTable;
use Boxberry\Bitrix\Helpers\BoxSizesCalculator;
use Boxberry\Bitrix\Helpers\Delivery;
use Boxberry\Bitrix\Helpers\Logger;
use Boxberry\Bitrix\Helpers\Options;
use Bitrix\Main\Localization\Loc;

Loc::loadMessages(__FILE__);

class Order
{
    public const BB_TRACKING_URL = 'https://boxberry.ru/tracking-page?id=';
    private BitrixOrder $order;
    private OrderData $orderData;
    private ShipmentCollection $shipmentCollection;
    private Options $options;
    private Result $result;
    private bool $isEventLogEnabled;

    public function __construct(BitrixOrder $order)
    {
        $this->order = $order;
        $this->result = new Result();
        $this->isEventLogEnabled = Options::isEventLogEnabled();

        try {
            $this->shipmentCollection = $order->getShipmentCollection();
            $this->orderData = new OrderData();
            $this->orderData->getValues($order->getId());
            $this->options = new Options($this->orderData->getDeliveryId() ?: $this->order->getField('DELIVERY_ID'));
        } catch (\Exception $e) {
            if ($this->isEventLogEnabled) {
                Logger::Add($e->getMessage(), __METHOD__ . '();', $e);
            }

            $this->result->addError(new Error($e->getMessage()));
        }

    }

    public function create(): Result
    {
        if (!$this->orderData->getOrderId()) {
            $createOrder = \Boxberry\Bitrix\Handlers\Order::saveOrderInBbOrdersTable($this->order);

            if (!$createOrder->isSuccess()) {
                if ($this->isEventLogEnabled) {
                    Logger::Add($createOrder->getErrorMessages()[0], __METHOD__ . '();');
                }

                $this->result->addErrors($createOrder->getErrors());
                return $this->result;
            }
        }

        $client = new Client($this->options->getApiKey());

        $bbParcelParams = new ParcelParams();

        if ($this->orderData->getTrack() && !$this->orderData->getAct() && in_array($this->orderData->getBbStatusId(), ParcelParams::ORDER_UPDATE_AVAILABLE_STATUSES_ID)) {
            $bbParcelParams->setUpdateByTrack($this->orderData->getTrack());
        }

        $bbParcelParams->setOrderId($this->options->getOrderPrefix() . $this->orderData->getAccountNumber());
        $bbParcelParams->setPrice($this->orderData->getDeclaredCost());
        $bbParcelParams->setPaymentSum($this->options->isParselcreateForcePaymentTypeSet() ? $this->options->getParselcreateForcePaySum($this->orderData->getPaymentAmount()) : $this->orderData->getPaymentAmount());
        $bbParcelParams->setDeliverySum($this->orderData->getDeliveryCost());
        $bbParcelParams->setVid($this->orderData->getDeliveryType());

        if ($this->orderData->getDeliveryType() === ParcelParams::COURIER_VID_ID) {

            try {
                $city = CitiesFullTable::getBbCityByBitrixLocationCode($this->orderData->getCity());
            } catch (\Exception $e) {
                if ($this->isEventLogEnabled) {
                    Logger::Add($e->getMessage(), __METHOD__ . '();', $e);
                }

                $this->result->addError(new Error($e->getMessage()));
                return $this->result;
            }

            if ($city->getCountryCode() === CitiesFullTable::RU_COUNTRY_CODE) {
                $kurdost = new Kurdost();
                $kurdost->setIndex($this->orderData->getPostcode() ?: '');
                $kurdost->setCiti($city->getName() . ' ' . $city->getRegion());
                $kurdost->setAddressp($this->orderData->getAddress());
                $bbParcelParams->setKurdost($kurdost);
            } else {

                try {
                    $address = Client::daDataRequest($city->getName() . ' ' . $this->orderData->getAddress());
                } catch (\Exception $e) {
                    if ($this->isEventLogEnabled) {
                        Logger::Add($e->getMessage(), __METHOD__ . '();', $e);
                    }

                    $this->result->addError(new Error($e->getMessage()));
                    return $this->result;
                }

                $export = new Export();
                $export->setIndex(ParcelParams::EAEU_COURIER_DEFAULT_INDEX);
                $export->setCountryCode($city->getCountryCode());
                $export->setCityCode($city->getCode());
                $export->setArea($city->getRegion());
                $export->setStreet($address->getStreet());
                $export->setHouse($address->getHouse());
                $export->setFlat($address->getFlat());
                $export->setTransporterGuid(ParcelParams::TRANSPORTER_GUID);
                $bbParcelParams->setExport($export);
            }
        }

        $shop = new Shop();
        if ($this->orderData->getDeliveryType() !== ParcelParams::COURIER_VID_ID) {
            $shop->setName($this->orderData->getPickupPointCode());
        }
        $shop->setName1($this->orderData->getPointForParcel());
        $bbParcelParams->setShop($shop);

        $customer = new Customer();

        $customer->setEmail($this->orderData->getEmail());
        $customer->setPhone($this->orderData->getPhone());

        $payerTypeId = $this->options::getPayerTypeId($this->order->getPersonTypeId());

        if ($payerTypeId === Options::FIZ_PAYER_TYPE_ID) {
            $customer->setFio($this->orderData->getFullName());
        }

        if ($payerTypeId === Options::JUR_PAYER_TYPE_ID) {
            $customer->setFio($this->orderData->getContactPerson() ?: $this->orderData->getFullName());
            $customer->setName($this->orderData->getCompany());
        }

        $bbParcelParams->setCustomer($customer);
        $bbParcelParams->setItems($this->getItems());
        $bbParcelParams->setWeights($this->getWeights());

        if ($this->orderData->getIssueType() !== NULL) {
            $bbParcelParams->setIssue($this->orderData->getIssueType());
        } else {
            $bbParcelParams->setIssue($this->options->getIssueType());
        }

        try {
            $parselResult = $client->parselCreate($bbParcelParams);
            $this->setTrackingNumberToShipment($parselResult->getTrack());

            $this->result->setData([
                'message' => Loc::getMessage('ORDER') . $this->orderData->getAccountNumber() . ' ' . Loc::getMessage('SUCCESS') . ' ' . ($bbParcelParams->getUpdateByTrack() ? Loc::getMessage('UPDATED') : Loc::getMessage('CREATED')) . ' ' . Loc::getMessage('IN_LK_BOXBERRY'),
                'track' => $parselResult->getTrack(),
                'label' => $parselResult->getLabel(),
            ]);

            $this->orderData->setTrack($parselResult->getTrack());
            $this->orderData->setStickers($parselResult->getLabel());
            $this->orderData->setError('');

            $lastStatus = $client->getLastStatusData([$parselResult->getTrack()]);

            $this->orderData->setBbStatusId($lastStatus->getLastStatusIdByTrack($parselResult->getTrack()));
            $this->orderData->setBbStatusDate($lastStatus->getLastStatusDateByTrack($parselResult->getTrack()));
            $this->orderData->setBbStatusName($lastStatus->getLastStatusNameByTrack($parselResult->getTrack()));

            $this->orderData->setValues($this->order->getId());
        } catch (\Exception $e) {
            $this->orderData->setError($e->getMessage());
            $this->orderData->setValues($this->order->getId());

            if ($this->isEventLogEnabled) {
                Logger::Add($e->getMessage(), __METHOD__ . '();', $e);
            }

            $this->result->addError(new Error($e->getMessage()));
            return $this->result;

        }

        return $this->result;

    }

    public function getItems(): array
    {
        $items = $this->order->getBasket()->getBasketItems();
        $itemsArray = [];

        foreach ($items as $item) {
            $itemsArray[] = [
                'id' => $this->getArtNumberByBasketItem($item),
                'name' => $item->getField('NAME'),
                'nds' => !is_null($item->getField('VAT_RATE')) ? $item->getField('VAT_RATE') * 100 : '',
                'price' => $item->getField('PRICE'),
                'quantity' => intval(ceil($item->getField('QUANTITY'))),
            ];
        }

        return $itemsArray;
    }

    public function getWeights(): array
    {
        $weights = [];

        try {
            $boxes = $this->orderData->getPlaces() ? Json::decode($this->orderData->getPlaces()): BoxSizesCalculator::getBoxSizesArrayFromOrderItems($this->order->getBasket()->toArray(), $this->orderData->getDeliveryId());

            $event = new Event(Options::getModuleId(), 'onBeforeGetWeights', ['orderId' => $this->orderData->getOrderId(), 'boxes' => &$boxes]);
            $event->send();

            $this->orderData->setPlaces(Json::encode($boxes));

            $i = 1;
            foreach ($boxes as $box) {
                $weights = array_merge($weights, [
                    'weight' . ($i > 1 ? $i : '') => $box['Weight'],
                    'x' . ($i > 1 ? $i : '') => $box['Height'],
                    'y' . ($i > 1 ? $i : '') => $box['Width'],
                    'z' . ($i > 1 ? $i : '') => $box['Depth'],
                ]);
                $i++;
            }
        } catch (\Exception $e) {
            if ($this->isEventLogEnabled) {
                Logger::Add($e->getMessage(), __METHOD__ . '();', $e);
            }
        }


        return $weights;
    }

    private function setTrackingNumberToShipment($trackingNumber): void
    {
        $isEnabledSetTrackToShipment = Options::isEnabledSetTrackToShipment();
        $profileIds = Delivery::getAllProfilesIds();

        try {
            /** @var Shipment $shipment */
            foreach ($this->shipmentCollection as $shipment) {
                if (!$shipment->isSystem() && in_array($shipment->getDeliveryId(), $profileIds) && $isEnabledSetTrackToShipment) {
                    $shipment->setField('TRACKING_NUMBER', $trackingNumber);
                    $shipment->save();
                }
            }
        } catch (\Exception $e) {

            if ($this->isEventLogEnabled) {
                Logger::Add($e->getMessage(), __METHOD__ . '();', $e);
            }
        }

    }

    private function getArtNumberByBasketItem(BasketItem $item): string
    {
        if (!$artNumberPropCode = Options::getArtNumberPropCode()) {
            return $item->getProductId();
        }

        try {
            $propValues = $item->getPropertyCollection()->getPropertyValues();

            if (!empty($propValues[$artNumberPropCode]['VALUE'])) {
                return $propValues[$artNumberPropCode]['VALUE'];
            }
        } catch (\Exception $e) {
            if ($this->isEventLogEnabled) {
                Logger::Add($e->getMessage(), __METHOD__ . '();', $e);
            }

            return $item->getProductId();
        }

        return $item->getProductId();

    }

}