<?php

namespace Boxberry\Bitrix\Handlers;

use Bitrix\Main\Context;
use Bitrix\Main\Error;
use Bitrix\Main\Event;
use Bitrix\Main\EventResult;
use Bitrix\Main\Localization\Loc;
use Bitrix\Main\Result;
use Bitrix\Main\Text\StringHelper;
use Bitrix\Main\Type\Date;
use Bitrix\Sale\Delivery\Services\Manager;
use Bitrix\Sale\Payment;
use Bitrix\Sale\ResultError;
use Bitrix\Sale\Shipment;
use Boxberry\Api\Client;
use Boxberry\Api\Entity\ParcelParams;
use Boxberry\Bitrix\Agents\Tracking;
use Boxberry\Bitrix\CitiesFullTable;
use Boxberry\Bitrix\Entity\OrderData;
use Boxberry\Bitrix\Forms\OrderForm;
use Boxberry\Bitrix\Helpers\BoxSizesCalculator;
use Boxberry\Bitrix\Helpers\Delivery;
use Boxberry\Bitrix\Helpers\Extensions;
use Boxberry\Bitrix\Helpers\Logger;
use Boxberry\Bitrix\Helpers\Options;
use Boxberry\Bitrix\Helpers\Validator;
use Boxberry\Bitrix\Helpers\Widget;
use Boxberry\Bitrix\OrdersTable;
use Boxberry\Bitrix\PointsForParcelsTable;
use Sale\Handlers\Delivery\BoxberryProfile;
use Bitrix\Sale\Order as BitrixOrder;

Loc::loadMessages(__FILE__);

class Order
{
    public static function checkOrderBeforeSaved(Event $event): EventResult
    {
        /** @var BitrixOrder $order */
        $order = $event->getParameter('ENTITY');
        $deliveryId = $order->getField('DELIVERY_ID');
        try {
            $options = new Options($deliveryId);
        } catch (\Exception $e) {
            return new EventResult(EventResult::SUCCESS);
        }

        $widget = new Widget();

        if($options->isCheckPvzIsSelected() && self::isPvzCheckNeeded()) {
            if (in_array($deliveryId, Delivery::getPvzProfilesIds()) && !$widget->isPvzIdSet()) {
                return
                    new EventResult(
                        EventResult::ERROR,
                        new ResultError(Loc::getMessage('SELECT_PVZ_WARNING')),
                        'sale'
                    );

            }
            return new EventResult(EventResult::SUCCESS);
        }

        return new EventResult(EventResult::SUCCESS);

    }

    public static function saveOrderInBbOrdersTable(BitrixOrder $order): Result
    {
        $result = new Result();
        $isEventLogEnabled = Options::isEventLogEnabled();

        try {
            $orderProps = $order->getPropertyCollection();
            $basket = $order->getBasket();
            $options = new Options($order->getField('DELIVERY_ID'));
        } catch (\Exception $e) {
            if ($isEventLogEnabled) {
                Logger::Add($e->getMessage(), __METHOD__ . '();', $e);
            }

            $result->addError(new ResultError($e->getMessage()));
            return $result;
        }

        $widget = new Widget();
        $orderData = new OrderData();

        try {
            $orderData->getValues($order->getId());

            if (!$orderData->getPointForParcel()) {
                $orderData->setPointForParcel($options->getPointForParcelCode());
            }
        } catch (\Exception $e) {
        }

        $payerType = Options::getPayerTypeId($order->getPersonTypeId());

        $orderData->setSiteId($order->getSiteId());
        $orderData->setOrderId($order->getId());
        $orderData->setAccountNumber($order->getField('ACCOUNT_NUMBER'));
        $orderData->setDeliveryId($order->getField('DELIVERY_ID'));
        $orderData->setParentDeliveryId($options->getParentId());
        $orderData->setBitrixStatusId($order->getField('STATUS_ID'));

        try {
            $orderData->setDeclaredCost($basket->getPrice());
            $orderData->setPaymentAmount((($order->getField('SUM_PAID') >= $basket->getPrice()) || $order->isPaid()) ? 0 :  $basket->getPrice() +  $order->getField('PRICE_DELIVERY') - $order->getField('SUM_PAID'));
        } catch (\Exception $e) {
            if ($isEventLogEnabled) {
                Logger::Add($e->getMessage(), __METHOD__ . '();', $e);
            }
        }

        $orderData->setDeliveryCost($order->getField('PRICE_DELIVERY'));
        $orderData->setDeliveryType($options->getProfileType());
        $orderData->setPostcode(Delivery::getPropValue($orderProps->getItemByOrderPropertyCode($options->getZip())));
        $orderData->setCity(Delivery::getPropValue($orderProps->getItemByOrderPropertyCode($options->getLocation())));
        $orderData->setAddress(Delivery::getAddress($orderProps, $options));
        $orderData->setPickupPointCode(($options->isPickup() && self::isPvzCheckNeeded()) ? $widget->getPvzId() : null);

        if ($payerType === Options::FIZ_PAYER_TYPE_ID) {
            $orderData->setFullName(Delivery::getFio($orderProps, $options));
        }

        if ($payerType === Options::JUR_PAYER_TYPE_ID) {
            $orderData->setContactPerson(Delivery::getPropValue($orderProps->getItemByOrderPropertyCode($options->getContactPerson())));
            $orderData->setCompany(Delivery::getPropValue($orderProps->getItemByOrderPropertyCode($options->getCompany())));
        }

        $orderData->setPhone(Delivery::getPropValue($orderProps->getItemByOrderPropertyCode($options->getPhone())));
        $orderData->setEmail(Delivery::getPropValue($orderProps->getItemByOrderPropertyCode($options->getEmail())));
        $orderData->setIssueType($orderData->getIssueType());

        try {
            $orderData->setValues($order->getId());
        } catch (\Exception $e) {
            if ($isEventLogEnabled) {
                Logger::Add($e->getMessage(), __METHOD__ . '();', $e);
            }

            $result->addError(new ResultError($e->getMessage()));
            return $result;
        }

        return $result;
    }

    public static function createBbOrderOnSaveEvent(Event $event): void
    {
        /** @var BitrixOrder $order */
        if (!$order = $event->getParameter('ENTITY')) {
            return;
        }

        if (!Validator::isBoxberryDeliveryInOrder($order)) {
            return;
        }

        self::saveOrderInBbOrdersTable($order);

    }

    public static function isPvzCheckNeeded(): bool
    {
        $request = Context::getCurrent()->getRequest();
        return !$request->isAdminSection() && $request->isAjaxRequest();
    }

    public static function isAdminSection(): bool
    {
        return Context::getCurrent()->getRequest()->isAdminSection();
    }

    public static function changeBitrixOrderStatus($orderId, $statusToChange, $bbStatusId): Result
    {
        $result = new Result();

        try {
            $order = BitrixOrder::load($orderId);
            $order->setField('STATUS_ID', $statusToChange);

            if (Options::isSetOrderPaidWhenDelivered() && $bbStatusId == Tracking::BB_SUCCESS_FINAL_STATUS_ID) {
                $paymentCollection = $order->getPaymentCollection();
                foreach ($paymentCollection as $payment) {
                    /** @var Payment $payment */

                    if ($payment->isPaid()) {
                        continue;
                    }
                    $payment->setPaid('Y');
                }
            }

            $order->save();

        } catch (\Exception $e) {
            if (Options::isEventLogEnabled()) {
                Logger::Add($e->getMessage(), __METHOD__ . '();', $e);
            }

            $result->addError(new Error($e->getMessage()));
            return $result;
        }

        return $result;
    }

    public static function createBbOrderOnBitrixStatusChange(Event $event): void
    {
        /** @var BitrixOrder $order */
        $order = $event->getParameter('ENTITY');

        if (!Validator::isBoxberryDeliveryInOrder($order)) {
            return;
        }

        if ($order->getField('STATUS_ID') !== Options::getOrderStatusForExport()) {
            return;
        }

        (new \Boxberry\Bitrix\Entity\Order($order))->create();

    }

    public static function runParselCreate($orderId): Result
    {
        try {
            $order = BitrixOrder::load($orderId);
        } catch (\Exception $e) {
            $result = new Result();
            $result->addError(new Error($e->getMessage()));
            return $result;
        }

        return (new \Boxberry\Bitrix\Entity\Order($order))->create();

    }

    public static function addBbOrderInfoBlock(Event $event): EventResult
    {
        $order = $event->getParameter('ORDER');
        if (!Options::isEnabledCreateOrderAdminForAll() && !Validator::isBoxberryDeliveryInOrder($order)) {
            return new EventResult(EventResult::SUCCESS);
        }

        $request = Context::getCurrent()->getRequest();
        $boxberryHtmlName = '<span style="color: #ED1651;">Boxberry</span>';

        try {
            $orderData = new OrderData();
            $orderData->getValues($order->getId());

            if ($request->isAdminSection() && (str_contains($request->getRequestUri(), 'sale_order_view') || str_contains($request->getRequestUri(), 'sale_order_edit'))) {
                if (str_contains($request->getRequestUri(), 'sale_order_view')) {
                    Extensions::initOrder();
                    (new OrderForm($order))->show();
                }

                $infoBlockShowArray = self::getInfoBlockShowArray($orderData, $boxberryHtmlName, $request);
                return new EventResult(EventResult::SUCCESS, $infoBlockShowArray);
            }

        } catch (\Exception $e) {
            return new EventResult(EventResult::SUCCESS);
        }

        return new EventResult(EventResult::SUCCESS);
    }

    private static function getInfoBlockShowArray($orderData, $boxberryHtmlName, $request): array
    {
        $infoBlockShowArray = [];

        if ($request->isAdminSection() && str_contains($request->getRequestUri(), 'sale_order_view')) {
            $infoBlockShowArray = self::getInfoBlockForView($orderData, $boxberryHtmlName);
        }

        if ($request->isAdminSection() && str_contains($request->getRequestUri(), 'sale_order_edit')) {
            $infoBlockShowArray = self::getInfoBlockForEdit($orderData, $boxberryHtmlName);
        }

        return $infoBlockShowArray;
    }

    private static function getInfoBlockForView($orderData, $boxberryHtmlName): array
    {
        return [
            [
                'TITLE' => Loc::getMessage('ORDER_STATUS_IN') . $boxberryHtmlName . ':',
                'VALUE' => !empty($orderData->getBbStatusName()) ? $orderData->getBbStatusName() : ' ',
                'ID' => 'bb_status'
            ],
            [
                'TITLE' => Loc::getMessage('TRACK_NUMBER_IN') . $boxberryHtmlName . ':',
                'VALUE' => !empty($orderData->getTrack()) ? $orderData->getTrack() : ' ',
                'ID' => 'bb_track'
            ],
            [
                'TITLE' => Loc::getMessage('STATUS_DATE_CHANGE_IN') . $boxberryHtmlName . ':',
                'VALUE' => !empty($orderData->getBbStatusDate()) ? $orderData->getBbStatusDate() : ' ',
                'ID' => 'bb_status_date'
            ],
            [
                'TITLE' => Loc::getMessage('STICKER_URL'),
                'VALUE' => '<a href="' . $orderData->getStickers() . '" target="_blank">' . Loc::getMessage('DOWNLOAD') . '</a>',
                'ID' => 'bb_stickers_url'
            ],
            [
                'TITLE' => Loc::getMessage('ACT_URL'),
                'VALUE' => '<a href="' . $orderData->getAct() . '" target="_blank">' . Loc::getMessage('DOWNLOAD') . '</a>',
                'ID' => 'bb_act_url'
            ]
        ];
    }

    private static function getInfoBlockForEdit($orderData, $boxberryHtmlName): array
    {
        $infoBlockShowArray = [];

        if (!empty($orderData->getBbStatusName())) {
            $infoBlockShowArray[] = [
                'TITLE' => Loc::getMessage('ORDER_STATUS_IN') . $boxberryHtmlName . ':',
                'VALUE' => $orderData->getBbStatusName(),
                'ID' => 'bb_status'
            ];
        }

        if (!empty($orderData->getTrack())) {
            $infoBlockShowArray[] = [
                'TITLE' => Loc::getMessage('TRACK_NUMBER_IN') . $boxberryHtmlName . ':',
                'VALUE' => $orderData->getTrack(),
                'ID' => 'bb_track'
            ];
        }

        if (!empty($orderData->getBbStatusDate())) {
            $infoBlockShowArray[] = [
                'TITLE' => Loc::getMessage('STATUS_DATE_CHANGE_IN') . $boxberryHtmlName . ':',
                'VALUE' => $orderData->getBbStatusDate(),
                'ID' => 'bb_status_date'
            ];
        }

        if (!empty($orderData->getStickers())) {
            $infoBlockShowArray[] = [
                'TITLE' => Loc::getMessage('STICKER_URL'),
                'VALUE' => '<a href="' . $orderData->getStickers() . '" target="_blank">' . Loc::getMessage('DOWNLOAD') . '</a>',
                'ID' => 'bb_stickers_url'
            ];
        }

        if (!empty($orderData->getAct())) {
            $infoBlockShowArray[] = [
                'TITLE' => Loc::getMessage('ACT_URL'),
                'VALUE' => '<a href="' . $orderData->getAct() . '" target="_blank">' . Loc::getMessage('DOWNLOAD') . '</a>',
                'ID' => 'bb_act_url'
            ];
        }

        return $infoBlockShowArray;
    }

    public static function addBbOrderJsData(&$arResult, &$arParams): void
    {
        if (isset($arResult['JS_DATA']['DELIVERY']) && is_array($arResult['JS_DATA']['DELIVERY'])) {

            $widget = new Widget();
            $widget->setCurrentId('');

            foreach ($arResult['JS_DATA']['DELIVERY'] as $delivery) {
               if (isset($delivery['CHECKED']) && $delivery['CHECKED'] === 'Y') {
                     $widget->setCurrentId($delivery['ID']);
                   break;
               }
            }
        }
    }

    public static function getOrderData($orderId, $deliveryId): Result
    {
        $result = new Result();

        try {
            $orderData = new OrderData();
            $orderData->getValues($orderId);
            $options = new Options($deliveryId);
            $order = BitrixOrder::load($orderId);
            $orderProps = $order->getPropertyCollection();

            $locationCode = Delivery::getPropValue($orderProps->getItemByOrderPropertyCode($options->getLocation()));
            $city = CitiesFullTable::getBbCityByBitrixLocationCode($locationCode);
            $orderNumber = $order->getField('ACCOUNT_NUMBER');

            $fio = $orderData->getFullName() ?: Delivery::getFio($orderProps, $options);

            if ($orderData->getPhone()) {
                $phone = $orderData->getPhone();
            } else {
                $phone = Delivery::getPropValue($orderProps->getItemByOrderPropertyCode($options->getPhone()));
            }

            if ($orderData->getEmail()) {
                $email = $orderData->getEmail();
            } else {
                $email = Delivery::getPropValue($orderProps->getItemByOrderPropertyCode($options->getEmail()));
            }

            if ($orderData->getContactPerson()) {
                $contactPerson = $orderData->getContactPerson();
            } else {
                $contactPerson = Delivery::getPropValue($orderProps->getItemByOrderPropertyCode($options->getContactPerson()));
            }

            if ($orderData->getCompany()) {
                $company = $orderData->getCompany();
            } else {
                $company = Delivery::getPropValue($orderProps->getItemByOrderPropertyCode($options->getCompany()));
            }

            if ($orderData->getPostcode()) {
                $postcode = $orderData->getPostcode();
            } else {
                $postcode = Delivery::getPropValue($orderProps->getItemByOrderPropertyCode($options->getZip()));
            }

            $pickupPointCode = $orderData->getPickupPointCode() ?: '';

            $address = $orderData->getAddress() ?: Delivery::getAddress($orderProps, $options);

            if ($orderData->getPaymentAmount() !== null) {
                $paymentAmount = $orderData->getPaymentAmount();
            } else {
                $paymentAmount = $order->getField('PRICE');
            }

            if ($orderData->getDeclaredCost() !== null) {
                $declaredCost = $orderData->getDeclaredCost();
            } else {
                $declaredCost = $order->getBasket()->getPrice();
            }

            if ($orderData->getDeliveryCost() !== null) {
                $deliveryCost = $orderData->getDeliveryCost();
            } else {
                $deliveryCost = $order->getField('PRICE_DELIVERY');
            }

            $pointForParcel = $orderData->getPointForParcel() ? PointsForParcelsTable::getPointNameByCode($orderData->getPointForParcel()) : PointsForParcelsTable::getPointNameByCode($options->getPointForParcelCode());
            $issueType = $orderData->getIssueType() ?: $options->getIssueType();

            $result->setData([
                'message' => Loc::getMessage('GET_ORDER_DATA_SUCCESS_MESSAGE'),
                'PAYMENT_AMOUNT' => $paymentAmount,
                'ACCOUNT_NUMBER' => $orderNumber,
                'DECLARED_COST' => $declaredCost,
                'DELIVERY_COST' => $deliveryCost,
                'FULL_NAME' => $fio,
                'PHONE' => $phone,
                'EMAIL' => $email,
                'COMPANY' => $company,
                'CONTACT_PERSON' => $contactPerson,
                'WIDGET_KEY' => $options->isTestMode() ? Client::TEST_WIDGET_KEY : $options->getWidgetKey(),
                'DELIVERY_TYPE' => $options->getProfileType(),
                'POSTCODE' => $postcode,
                'PICKUP_POINT_CODE' => $pickupPointCode,
                'ADDRESS' => $address,
                'PARENT_DELIVERY_ID' => $options->getParentId() ?: '',
                'DELIVERY_ID' => $options->getDeliveryId() ?: '',
                'ISSUE_TYPE' => $issueType,
                'CITY_NAME' => $city->getName() . ' ' . $city->getRegion(),
                'CITY' => $locationCode,
                'POINT_FOR_PARCEL' => $pointForParcel,
                'BITRIX_STATUS_ID' => $order->getField('STATUS_ID'),
                'SITE_ID' => $order->getSiteId(),
            ]);

            return $result;

        } catch (\Exception $e) {
            $result->addError(new Error($e->getMessage()));
            return $result;
        }
    }

    public static function updateOrderDataFromAdmin($orderId, $data): Result
    {
        $orderData = new OrderData();
        $result = new Result();

        try {
            $options = new Options($data['deliveryId']);

            $orderData->setOrderId($orderId);
            $orderData->setParentDeliveryId($options->getParentId());

            foreach ($data as $key => $value) {
                $propName = lcfirst(StringHelper::snake2camel($key));
                if (method_exists($orderData, 'set' . $propName)) {
                    $orderData->{'set' . $propName}($value);
                }
            }
        } catch (\Exception $e) {
            $result->addError(new Error($e->getMessage()));
            return $result;
        }

        try {
            $orderData->setValues($orderId) ? $result->setData(['message' => Loc::getMessage('UPDATE_ORDER_DATA_FROM_ADMIN_SUCCESS_MESSAGE')]) : $result->addError(new Error(Loc::getMessage('UPDATE_ORDER_DATA_FROM_ADMIN_ERROR_MESSAGE')));
            return $result;
        } catch (\Exception $e) {
            $result->addError(new Error($e->getMessage()));
            return $result;
        }
    }

    public static function getOrderTrackingData($orderId): Result
    {
        $orderData = new OrderData();
        $result = new Result();
        $storageDate = new Date();
        $date = '';

        try {
            $orderData->getValues($orderId);
            $options = new Options($orderData->getDeliveryId());
        } catch (\Exception $e) {
            $result->addError(new Error($e->getMessage(), 0, ['orderId' => $orderId, 'deleteAvailable' => false, 'cancelAvailable' => false]));
            return $result;
        }

        if (!$orderData->getTrack()) {
            $result->addError(new Error(Loc::getMessage('GET_ORDER_TRACKING_DATA_TRACK_NOT_FOUND'), 0, ['orderId' => $orderId, 'deleteAvailable' => false, 'cancelAvailable' => false]));
            return $result;
        }

        $client = new Client($options->getApiKey());

        try {
            $getLastStatusData = $client->getLastStatusData([$orderData->getTrack()]);

            $date = $getLastStatusData->getLastStatusStoreDateByTrack($orderData->getTrack());

            if ($date) {
                list($year, $month, $day) = explode('-', $date);
                $storageDate->setDate((int)$year, (int)$month, (int)$day);
            }

            $orderData->setBbStatusId($getLastStatusData->getLastStatusIdByTrack($orderData->getTrack()));
            $orderData->setBbStatusName($getLastStatusData->getLastStatusNameByTrack($orderData->getTrack()));
            $orderData->setBbStatusDate($getLastStatusData->getLastStatusDateByTrack($orderData->getTrack()));
            $orderData->setValues($orderId);
        } catch (\Exception $e) {
            $result->addError(new Error($e->getMessage(), 0, [
                'orderId' => $orderId,
                'deleteAvailable' => false,
                'cancelAvailable' => false
            ]));
        }

        if (!$result->isSuccess() && !$orderData->getBbStatusId()) {
            return $result;
        }

        $result->setData([
            'message' => Loc::getMessage('GET_ORDER_TRACKING_DATA_SUCCESS_MESSAGE'),
            'bb_track' => $orderData->getTrack(),
            'bb_status_id' => $orderData->getBbStatusId(),
            'bb_status_date' => $orderData->getBbStatusDate(),
            'bb_status' => $orderData->getBbStatusName(),
            'bb_stickers_url' => $orderData->getStickers(),
            'bb_act_url' => $orderData->getAct(),
            'bb_storage_date' => Date::isCorrect($date, 'Y-m-d') ? $storageDate->format('d.m.Y') : '',
            'deleteAvailable' => in_array($orderData->getBbStatusId(), ParcelParams::ORDER_DELETE_AVAILABLE_STATUSES_ID),
            'cancelAvailable' => in_array($orderData->getBbStatusId(), ParcelParams::ORDER_CANCEL_AVAILABLE_STATUSES_ID),
            'changeStorageDateAvailable' => in_array($orderData->getBbStatusId(), ParcelParams::ORDER_CHANGE_STORAGE_DATE_AVAILABLE_STATUSES_ID)
        ]);

        return $result;
    }

    public static function changeStorageDateFromAdmin($date, $trackNumber, $deliveryId): Result
    {
        $result = new Result();

        try {
            $options = new Options($deliveryId);
        } catch (\Exception $e) {
            $result->addError(new Error($e->getMessage()));
            return $result;
        }

        $client = new Client($options->getApiKey());

        try {
            $changeStorageDateResult = $client->changeOrderStorageDate($date, $trackNumber);

            if (isset($changeStorageDateResult['err']) && $changeStorageDateResult['err'] !== false) {
                $result->addError(new Error($changeStorageDateResult['err']));
                return $result;
            }

            $result->setData(['message' => Loc::getMessage('ORDER_STORAGE_DATE_CHANGED_SUCCESS')]);
        } catch (\Exception $e) {
            $result->addError(new Error($e->getMessage()));
            return $result;
        }

        return $result;

    }

    public static function calculateDeliveryCostFromAdmin($orderId, $deliveryId, $data = []): Result
    {
        $result = new Result();

        try {
            $options = new Options($deliveryId);
        } catch (\Exception $e) {
            Logger::Add($e->getMessage(), __METHOD__ . '();', $e);
            $result->addError(new Error($e->getMessage()));
            return $result;
        }

        try {
            /** @var BitrixOrder $order */
            $order = BitrixOrder::load($orderId);

            /** @var BoxberryProfile $delivery */
            $delivery = Manager::getObjectById($deliveryId);
            $shipmentCollection = $order->getShipmentCollection();

            $calcExternalData = new Result();
            empty($data) ? $calcExternalData->addError(new Error(Loc::getMessage('CALC_FROM_ADMIN_EMPTY_EXTERNAL_DATA_ERROR'))) : $calcExternalData->setData($data);

            /** @var Shipment $shipment */
            $shipment = $shipmentCollection->createItem($delivery);
            $deliveryCost = Delivery::calculate($shipment, false, $calcExternalData);

            if (!$deliveryCost->isSuccess()) {
                $result->addError(new Error($deliveryCost->getErrorMessages()[0]));
                return $result;
            }

            $result->setData([
                'message' => Loc::getMessage('CALC_FROM_ADMIN_SUCCESS_MESSAGE'),
                'price' => $deliveryCost->getDeliveryPrice(),
                'period' => $deliveryCost->getPeriodDescription(),
                'deliveryType' => $options->getProfileType(),
            ]);

        } catch (\Exception $e) {
            $result->addError(new Error($e->getMessage()));
            if (Options::isEventLogEnabled()) {
                Logger::Add($e->getMessage(), __METHOD__ . '();', $e);
            }
            return $result;
        }

        return $result;
    }

    public static function cancelOrder($orderId, $cancelType = '1'): Result
    {
        $result = new Result();
        $orderData = new OrderData();

        try {
            $orderData->getValues($orderId);
        } catch (\Exception $e) {
            $result->addError(new Error($e->getMessage()));

            return $result;
        }

        if (!$orderData->getBbStatusId() && !$orderData->getTrack()) {
            $result->addError(new Error(Loc::getMessage('GET_ORDER_TRACKING_DATA_TRACK_NOT_FOUND')));

            return $result;
        }

        if (in_array($orderData->getBbStatusId(), ParcelParams::ORDER_CANCEL_AVAILABLE_STATUSES_ID) || in_array($orderData->getBbStatusId(), ParcelParams::ORDER_DELETE_AVAILABLE_STATUSES_ID)) {
            try {
                $options = new Options($orderData->getDeliveryId());
            } catch (\Exception $e) {
                $result->addError(new Error($e->getMessage()));

                return $result;
            }

            try {
                $client = new Client($options->getApiKey());
                $cancelOrderResult = $client->cancelOrder($orderData->getTrack(), $cancelType);

                if (isset($cancelOrderResult['err']) && $cancelOrderResult['err'] !== false && $cancelOrderResult['err'] !== Loc::getMessage('CANCEL_ORDER_NOT_FOUND')) {
                    $result->addError(new Error($cancelOrderResult['err']));
                    return $result;
                }

                if ($cancelType == '1') {
                    $orderData->setBbStatusId(0);
                    $orderData->setBbStatusDate('');
                    $orderData->setBbStatusName('');
                    $orderData->setTrack('');
                    $orderData->setStickers('');
                    $orderData->setAct('');
                    $orderData->setError('');
                }

                if ($orderData->setValues($orderId)) {
                    $result->setData(['message' => Loc::getMessage('ORDER_CANCEL_SUCCESS') . ($cancelType == '1' ? Loc::getMessage('ORDER_DELETED') : Loc::getMessage('ORDER_CANCELED'))]);
                    return $result;
                }

            } catch (\Exception $e) {
                $result->addError(new Error($e->getMessage()));
                return $result;
            }

        }

        $result->addError(new Error(($cancelType == '1' ? Loc::getMessage('ORDER_DELETE') : Loc::getMessage('ORDER_CANCEL')) . Loc::getMessage('AT_CURRENT_STATUS_NOT_AVAILABLE')));
        return $result;
    }

    public static function addOnAdminListDisplayMenu(&$list): void
    {
        if (isset($GLOBALS['APPLICATION']) && $GLOBALS['APPLICATION']->GetCurPage() === '/bitrix/admin/sale_order.php' && isset($list->arActions, $list->table_id) && $list->table_id === 'tbl_sale_order') {
            $list->arActions['bbParselCreate'] = [
                'action' => 'bbOrdersListHandler("parselCreate")',
                'value' => 'parselCreate',
                'name' => 'Boxberry: ' . Loc::getMessage('PRINT_STICKERS'),
            ];
            $list->arActions['bbParselSend'] = [
                'action' => 'bbOrdersListHandler("parselSend")',
                'value' => 'parselSend',
                'name' => 'Boxberry: ' . Loc::getMessage('PRINT_ACTS'),
            ];
            $list->arActions['bbParselCreateAndSend'] = [
                'action' => 'bbOrdersListHandler("parselCreateAndSend")',
                'value' => 'parselCreateAndSend',
                'name' => 'Boxberry: ' . Loc::getMessage('PRINT_ALL'),
            ];
        }

    }

    public static function handleOnEpilog(): void
    {
        if (isset($GLOBALS['APPLICATION']) && $GLOBALS['APPLICATION']->GetCurPage() === '/bitrix/admin/sale_order.php') {
            Extensions::initOrdersList();
        }
    }

    public static function runParselSendMulti($orderIds): Result
    {
        $result = new Result();
        $actLinksAndOrderIds = [];
        $apiTokensAndTracks = OrdersTable::getApTokensAndTracksByOrderIds($orderIds);

        if (!$apiTokensAndTracks->isSuccess()) {
            return $apiTokensAndTracks;
        } else {
            $apiTokensAndTracks = $apiTokensAndTracks->getData();
        }

        foreach ($apiTokensAndTracks as $apiTokensAndTrack) {
            $client = new Client($apiTokensAndTrack['apiToken']);

            try {
                $parselSendResult = $client->parselSend($apiTokensAndTrack['tracks']);
                $actLinksAndOrderIds[] = [
                    'actUrl' => $parselSendResult->getLabel(),
                    'orderIds' => $apiTokensAndTrack['orderIds']
                ];

                $lastStatus = $client->getLastStatusData($apiTokensAndTrack['tracks']);

                $orderIds = OrdersTable::getOrdersIdsByTracks($apiTokensAndTrack['tracks'])->getData();

                foreach ($orderIds as $orderId) {
                    $orderData = new OrderData();
                    $orderData->getValues($orderId);
                    $orderData->setBbStatusId($lastStatus->getLastStatusIdByTrack($orderData->getTrack()));
                    $orderData->setBbStatusName($lastStatus->getLastStatusNameByTrack($orderData->getTrack()));
                    $orderData->setBbStatusDate($lastStatus->getLastStatusDateByTrack($orderData->getTrack()));
                    $orderData->setValues($orderId);
                }

            } catch (\Exception $e) {
                if ($notIncludedTracksArray = Client::getParselSendNotIncludedTracks($e->getMessage())) {
                    $notIncludedOrderIds = OrdersTable::getOrdersIdsByTracks($notIncludedTracksArray);

                    if ($notIncludedOrderIds->isSuccess()) {
                        $result->addError(new Error(Loc::getMessage('CAN_NOT_INCLUDE_ORDERS_IN_ONE_ACT') . implode(', ', $notIncludedOrderIds->getData()), 0, ['notIncludedOrderIds' => $notIncludedOrderIds->getData()]));
                        return $result;
                    } else {
                        return $notIncludedOrderIds;
                    }
                }

                $result->addError(new Error($e->getMessage(), 0, ['tracks' => $apiTokensAndTrack['tracks'], 'orderIds' => $apiTokensAndTrack['orderIds']]));
                return $result;
            }
        }

        $saveActToOrders = OrdersTable::saveActToOrders($actLinksAndOrderIds);

        if (!$saveActToOrders->isSuccess()) {
            return $saveActToOrders;
        }

        $actFilesPaths = [];

        foreach ($actLinksAndOrderIds as $actLinkAndOrderId) {
            $path = Client::downloadFile($actLinkAndOrderId['actUrl'], Client::makeActFileNameByOrderIds($actLinkAndOrderId['orderIds']))->getData()['path'];
            if (!$path) {
                $result->addError(new Error(Loc::getMessage('CAN_NOT_DOWNLOAD_ACT_ON_URL') . $actLinkAndOrderId['actUrl'], 0, ['actLink' => $actLinkAndOrderId['actUrl'], 'orderIds' => $actLinkAndOrderId['orderIds']]));
            } else {
                foreach ($actLinkAndOrderId['orderIds'] as $orderId) {
                    $actFilesPaths[$orderId] = $path;
                }
            }
        }

        $result->setData($actFilesPaths);

        return $result;

    }

    public static function getOrderPlaces($orderId, $deliveryId): Result
    {
        $result = new Result();
        $orderData = new OrderData();
        $places = [];
        $source = 'ParcelInfo';

        try {
            $orderData->getValues($orderId);

            if ($orderData->getTrack()) {
                $options = new Options($deliveryId);
                $client = new Client($options->getApiKey());

                $parcelInfo = $client->parcelInfo($orderData->getTrack());

                foreach ($parcelInfo->getWeights() as $weight) {
                    $places[] = [
                        'Weight' => $weight['weight'],
                        'Height' => $weight['x'],
                        'Width' => $weight['y'],
                        'Depth' => $weight['z'],
                    ];
                }
            } else {
                $basketItems = BitrixOrder::load($orderId)->getBasket()->toArray();
                $places = BoxSizesCalculator::getBoxSizesArrayFromOrderItems($basketItems, $deliveryId);
                $source = 'BoxSizesCalculator';
            }

            $result->setData([
                'message' => Loc::getMessage('ORDER_PLACES_CALCULATE_SUCCESS'),
                'source' => $source,
                'places' => $places,
            ]);
        } catch (\Exception $e) {
            $result->addError(new Error($e->getMessage()));
        }

        return $result;
    }

}