<?php

namespace Boxberry\Bitrix\Helpers;

use Bitrix\Main\Application;
use Bitrix\Main\ArgumentException;
use Bitrix\Main\ArgumentNullException;
use Bitrix\Main\Error;
use Bitrix\Main\Grid\Declension;
use Bitrix\Main\IO\Directory;
use Bitrix\Main\Localization\Loc;
use Bitrix\Main\NotImplementedException;
use Bitrix\Main\ObjectPropertyException;
use Bitrix\Main\ORM\Data\UpdateResult;
use Bitrix\Main\Result;
use Bitrix\Main\SystemException;
use Bitrix\Sale\Basket;
use Bitrix\Sale\BasketItem;
use Bitrix\Sale\Delivery\CalculationResult;
use Bitrix\Sale\Delivery\Services\Table;
use Bitrix\Sale\Location\LocationTable;
use Bitrix\Sale\Order;
use Bitrix\Sale\Payment;
use Bitrix\Sale\PaymentCollection;
use Bitrix\Sale\PropertyValueCollectionBase;
use Bitrix\Sale\Shipment;
use Boxberry\Api\Client;
use Boxberry\Api\CountriesSettings;
use Boxberry\Api\Entity\DeliveryCalculationParams;
use Boxberry\Api\Entity\DeliveryCalculationResult;
use Boxberry\Api\Exceptions\BoxberryApiException;
use Boxberry\Api\Entity\WidgetSettings;
use Boxberry\Bitrix\Entity\City;
use Boxberry\Bitrix\PointsForParcelsTable;
use Sale\Handlers\Delivery\BoxberryHandler;
use Sale\Handlers\Delivery\BoxberryProfile;
use Boxberry\Bitrix\CitiesFullTable;
use Boxberry\Bitrix\Handlers\Order as BoxberryOrderHandler;
use Boxberry\Bitrix\Helpers\BoxSizesCalculator;

Loc::loadMessages(__FILE__);

class Delivery
{
    public const BB_DEFAULT_WEIGHT = 1000;
    public const BB_HANDLER_CLASS_NAME = '\Sale\Handlers\Delivery\BoxberryHandler';
    public const BB_PROFILE_CLASS_NAME = '\Sale\Handlers\Delivery\BoxberryProfile';

    public static function getPvzProfilesIds(): array
    {
        $result = [];

        try {
            $query = Table::query()
                ->setSelect(['ID', 'CONFIG'])
                ->where('ACTIVE', 'Y')
                ->where('PARENT_ACTIVE', 'Y')
                ->where('CLASS_NAME', self::BB_PROFILE_CLASS_NAME);

            self::checkParentActive($query);

            $list = $query->exec()->fetchAll();
        } catch (\Exception $e) {
            return $result;
        }

        foreach ($list as $item) {
            if (isset($item['CONFIG']['TYPE']['PROFILE_TYPE']) && $item['CONFIG']['TYPE']['PROFILE_TYPE'] === '1') {
                $result[] = $item['ID'];
            }
        }

        return $result;
    }

    public static function getBbHandlersIds(): array
    {
        $result = [];

        try {
            $query = Table::query()
                ->addSelect('ID')
                ->where('CLASS_NAME', self::BB_HANDLER_CLASS_NAME);

            $list = $query->exec()->fetchAll();
        } catch (\Exception $e) {
            return $result;
        }

        foreach ($list as $item) {
            $result[] = $item['ID'];
        }

        return $result;
    }

    public static function getAllProfilesIds(): array
    {
        $result = [];

        try {
            $query = Table::query()
                ->addSelect('ID')
                ->where('ACTIVE', 'Y')
                ->where('PARENT_ACTIVE', 'Y')
                ->where('CLASS_NAME', self::BB_PROFILE_CLASS_NAME);

            self::checkParentActive($query);

            $list = $query->exec()->fetchAll();
        } catch (\Exception $e) {
            return $result;
        }

        foreach ($list as $item) {
            $result[] = $item['ID'];
        }

        return $result;
    }

    private static function checkParentActive($query): void
    {
        $tableName = Table::getTableName();
        $query->registerRuntimeField(
            'PARENT_ACTIVE',
            [
                'data_type' => 'string',
                'expression' => [
                    "(SELECT `ACTIVE` FROM `{$tableName}` WHERE `{$tableName}`.`ID` = %s)",
                    'PARENT_ID'
                ]
            ]
        );
    }

    private static function cleanLocationsCache(Result $result): void
    {
        try {
            LocationTable::getEntity()->cleanCache();
            CitiesFullTable::getEntity()->cleanCache();
        } catch (\Exception $e) {
            $result->addError(new Error($e->getMessage()));
        }
    }

    public static function clearCache(string $type = 'all'): Result
    {
        $result = new Result();
        $cacheFolder = Application::getDocumentRoot() . '/bitrix/cache/';

        if (!Directory::isDirectoryExists($cacheFolder)) {
            $result->addError(new Error(Loc::getMessage('DIRECTORY') . $cacheFolder . Loc::getMessage('NOT_FOUND')));
            return $result;
        }

        $directoriesToDelete = match ($type) {
            'all', 'api' => ['bb_api'],
            'calculator' => ['bb_api/delivery_calculation'],
            'settings' => ['bb_api/widget_settings'],
            'zips' => ['bb_api/zip_check'],
            default => [],
        };

        if ($type === 'locations' || $type === 'all') {
            self::cleanLocationsCache($result);
        }

        foreach ($directoriesToDelete as $directory) {
            Directory::deleteDirectory($cacheFolder . $directory);
            if (Directory::isDirectoryExists($cacheFolder . $directory)) {
                $result->addError(new Error(Loc::getMessage('CAN_NOT_CLEAR_CACHE') . $directory));
            }
        }

        $result->setData(['message' => Loc::getMessage('CACHE_IS_CLEARED')]);

        return $result;
    }

    public static function updateDeliveryData($apiToken = '', $updateDataType = 'all'): Result
    {
        $result = new Result();
        $apiToken = empty($apiToken) ? Client::getTestToken() : trim($apiToken);
        $client = new Client($apiToken);

        try {

            if ($apiToken !== Client::getTestToken()) {
                if (!$client->checkApiToken()->isSuccess()) {
                    $client = new Client(Client::getTestToken());
                }
            }

            if ($updateDataType === 'all' || $updateDataType === 'cities') {
                $listCitiesFull = $client->listCitiesFull();
                CitiesFullTable::addCities($listCitiesFull);
            }

            if ($updateDataType === 'all' || $updateDataType === 'pointsForParcels') {
                $listPointsForParcels = $client->pointsForParcels();
                PointsForParcelsTable::addPointsForParcels($listPointsForParcels);
            }

        } catch (\Exception $e) {

            if (Options::isEventLogEnabled()) {
                Logger::Add($e->getMessage(), __METHOD__ . '();', $e);
            }

            $result->addError(new Error($e->getMessage()));
            return $result;
        }

        $result->setData(['message' => Loc::getMessage('UPDATE_DELIVERY_DATA_SUCCESS')]);

        return $result;
    }

    public static function getEmptyDeliveryData(): Result
    {
        $result = new Result();

        if (CitiesFullTable::isEmpty()) {
            $result->addError(new Error('cities'));
        }

        if (PointsForParcelsTable::isEmpty()) {
            $result->addError(new Error('pointsForParcels'));
        }

        if ($result->isSuccess()) {
            $result->setData(['message' => Loc::getMessage('NO_EMPTY_DATA')]);
        }

        return $result;
    }

    public static function getProfilesIdsWithNames(): array
    {
        try {
            $query = Table::query()
                ->setSelect(['ID', 'NAME'])
                ->where('ACTIVE', 'Y')
                ->where('CLASS_NAME', self::BB_PROFILE_CLASS_NAME)
                ->where('PARENT_ACTIVE', 'Y');

            self::checkParentActive($query);

            $results = $query->exec()->fetchAll();

            if (empty($results)) {
                return ['0' => Loc::getMessage('NOT_SELECTED')];
            }

            $profiles = ['0' => Loc::getMessage('NOT_SELECTED')];
            foreach ($results as $result) {
                $profiles[$result['ID']] = $result['NAME'] . ' (ID ' . $result['ID'] . ')';
            }

            return $profiles;
        } catch (\Exception $e) {
            return ['0' => Loc::getMessage('NOT_SELECTED')];
        }
    }

    private static function getCurrentPaySystemId($paymentCollection): int
    {
        $paySystemId = 0;
        /** @var PaymentCollection $paymentCollection */
        foreach ($paymentCollection as $payment) {
            /** @var Payment $payment */
            $paySystemId = $payment->getPaymentSystemId();
        }
        return $paySystemId;
    }

    private static function getDeliveryPeriodFormat($days): string
    {
        $declension = new Declension(Loc::getMessage('DELIVERY_PERIOD_ONE'), Loc::getMessage('DELIVERY_PERIOD_FOUR'), Loc::getMessage('DELIVERY_PERIOD_FIVE'));

        return $days . ' ' . $declension->get($days);
    }

    public static function correctDeliveryDaysWithSettings(int $days, Options $options, WidgetSettings $widgetSettings = null): string
    {
        if (!$options->isCalcSourceProfile()) {
            if (isset($widgetSettings)) {
                if ($widgetSettings->isHideDeliveryDays()) {
                    return '';
                }

                $days = $days + $widgetSettings->getAddDeliveryDays();
            }

            return self::getDeliveryPeriodFormat($days);
        }

        if ($options->isHideDeliveryDays()) {
            return '';
        }

        return self::getDeliveryPeriodFormat($days + (int)$options->getAddDeliveryDay());
    }

    private static function correctPriceWithSettings(float $deliverySum, float $paySum, float $orderSum, Options $options, CalculationResult $result): CalculationResult
    {
        if (!$options->isCalcSourceProfile()) {
            $result->setDeliveryPrice($deliverySum);
            return $result;
        }

        $addon = (float)$options->getFixSum() +
            ($deliverySum * (float)$options->getPriceBasePercent() / 100) +
            ($orderSum * (float)$options->getOrderSumPercent() / 100) +
            ($paySum * (float)$options->getPaySumPercent() / 100);

        $deliverySum += $addon;

        $precision = (int)$options->getLength();

        if ($precision > 0) {
            $roundType = (int)$options->getRound();
            if (1 === $roundType) {
                if (1 === $precision) {
                    $deliverySum = round($deliverySum);
                }
                if (2 === $precision) {
                    $deliverySum = round($deliverySum / 10) * 10;
                }
                if (3 === $precision) {
                    $deliverySum = round($deliverySum / 100) * 100;
                }
            }
            if (2 === $roundType) {
                if (1 === $precision) {
                    $deliverySum = ceil($deliverySum);
                }
                if (2 === $precision) {
                    $deliverySum = ceil($deliverySum / 10) * 10;
                }
                if (3 === $precision) {
                    $deliverySum = ceil($deliverySum / 100) * 100;
                }
            }
        }

        if ($options->isEnableMinTariff() && $deliverySum < (float)$options->getMinTariff()) {
            $deliverySum = (float)$options->getMinTariff();
        }
        if ($options->isEnableMaxTariff() && $deliverySum > (float)$options->getMaxTariff()) {
            $deliverySum = (float)$options->getMaxTariff();
        }

        if ((float)$options->getRangeFirstCost() >= $orderSum && 2 === (int)$options->getRangeFirstCalcType()) {
            $deliverySum = (float)$options->getRangeFirstFixCost();
        } elseif ((float)$options->getRangeSecondCost() < $orderSum && 2 === (int)$options->getRangeSecondCalcType()) {
            $deliverySum = (float)$options->getRangeSecondFixCost();
        }

        $result->setDeliveryPrice($deliverySum);
        return $result;

    }

    public static function getFio(PropertyValueCollectionBase $orderProps, Options $options): string
    {
        $defaultFio = Loc::getMessage('DEFAULT_FIO');

        if ($options->isFioOneProp()) {
            $fioProp = $orderProps->getItemByOrderPropertyCode($options->getFio());
            $fio = $fioProp !== null ? $fioProp->getValue() : $defaultFio;
        } else {
            $lastNameProp = $orderProps->getItemByOrderPropertyCode($options->getSecondname());
            $firstNameProp = $orderProps->getItemByOrderPropertyCode($options->getFirstname());
            $secondNameProp = $orderProps->getItemByOrderPropertyCode($options->getMiddlename());

            $lastName = self::getPropValue($lastNameProp);
            $firstName = self::getPropValue($firstNameProp);
            $secondName = self::getPropValue($secondNameProp);

            $fio = $lastName . ' ' . $firstName . ' ' . $secondName;

            if (empty($fio)) {
                $fio = $defaultFio;
            }
        }

        return $fio;
    }

    public static function getAddress(PropertyValueCollectionBase $orderProps, Options $options): string
    {
        if ($options->isAddressOneProp()) {
            $addressProp = $orderProps->getItemByOrderPropertyCode($options->getAddress());
            $address = self::getPropValue($addressProp);
        } else {
            $streetProp = $orderProps->getItemByOrderPropertyCode($options->getStreet());
            $houseProp = $orderProps->getItemByOrderPropertyCode($options->getHouse());
            $flatProp = $orderProps->getItemByOrderPropertyCode($options->getFlat());

            $street = self::getPropValue($streetProp);
            $house = self::getPropValue($houseProp);
            $flat = self::getPropValue($flatProp);

            $address = $street . ' ' . $house . ' ' . $flat;
        }

        return $address;
    }

    private static function setAddressPropIdToWidget(Widget $widget, Options $options, PropertyValueCollectionBase $orderProps): void
    {
        $widget->setAddressPropId('');
        $widget->setStreetPropId('');
        $widget->setHousePropId('');
        $widget->setFlatPropId('');

        if ($options->isAddressOneProp()) {
            $addressProp = $orderProps->getItemByOrderPropertyCode($options->getAddress());
            if ($addressProp !== null) {
                $widget->setAddressPropId($addressProp->getPropertyId());
            }
        } else {
            $streetProp = $orderProps->getItemByOrderPropertyCode($options->getStreet());
            $houseProp = $orderProps->getItemByOrderPropertyCode($options->getHouse());
            $flatProp = $orderProps->getItemByOrderPropertyCode($options->getFlat());

            if ($streetProp !== null) {
                $widget->setStreetPropId($streetProp->getPropertyId());
            }

            if ($houseProp !== null) {
                $widget->setHousePropId($houseProp->getPropertyId());
            }

            if ($flatProp !== null) {
                $widget->setFlatPropId($flatProp->getPropertyId());
            }
        }
    }

    private static function setWidgetCustomCity(Widget $widget, City $city): void
    {
        if ($city->getCountryCode() === CitiesFullTable::TJ_COUNTRY_CODE) {
            $widget->setCustomCity($city->getName());
        } else {
            $widget->setCustomCity($city->getName() . ' ' . $city->getRegion());
        }
    }

    public static function getPropValue($property): string
    {
        if (is_null($property) || is_null($property->getValue())) {
            return '';
        }

        return $property->getValue();
    }

    private static function getPaySum($price, Options $options, PaymentCollection $paymentCollection): float
    {
        if ($options->isForcePaymentTypeSet()) {
            return $options->getForcePaySum($price);
        }

        return $options->isPaySystemCod(self::getCurrentPaySystemId($paymentCollection)) ? $price : 0;
    }

    /**
     * Выполняет расчет стоимости доставки, учитывая все настройки/надбавки
     * @param Shipment $shipment - отгрузка
     * @param bool $checkCompatible - проверка совместимости для метода isCompatible
     * @param Result|null $externalData - внешние данные для расчета, если не переданы, то берутся из отгрузки
     * @param int $deliveryId - ID профиля доставки, если не передан, то берется из отгрузки
     * @return CalculationResult
     * @throws ArgumentException
     * @throws ArgumentNullException
     * @throws NotImplementedException
     * @throws ObjectPropertyException
     * @throws SystemException
     */
    public static function calculate(Shipment $shipment, bool $checkCompatible = false, Result $externalData = null, int $deliveryId = 0): CalculationResult
    {
        $result = new CalculationResult();
        $isEventLogEnabled = Options::isEventLogEnabled();
        $deliveryId = $deliveryId ?: $shipment->getDeliveryId();

        try {
            $options = new Options($deliveryId);
        } catch (\Exception $e) {
            if ($isEventLogEnabled) {
                Logger::Add($e->getMessage(), __METHOD__ . '();', $e);
            }

            $result->addError(new Error($e->getMessage()));
            return $result;
        }

        if (!$options->getWidgetKey() && !$options->isTestMode()) {
            $errorMessage = Loc::getMessage('WIDGET_KEY_IS_EMPTY');

            if ($isEventLogEnabled) {
                Logger::Add($errorMessage, __METHOD__ . '();');
            }
            $result->addError(new Error($errorMessage));
            return $result;
        }

        $token = $options->isTestMode() ? Client::getTestToken() : $options->getApiKey();
        $api = new Client($token);

        if (!$order = $shipment->getOrder()) {
            $errorMessage = Loc::getMessage('GET_ORDER_IN_CALCULATE_ERROR');

            if ($isEventLogEnabled) {
                Logger::Add($errorMessage, __METHOD__ . '();');
            }
            $result->addError(new Error($errorMessage));
            return $result;
        }

        if (!$basket = $order->getBasket()) {
            $errorMessage = Loc::getMessage('GET_BASKET_IN_CALCULATE_ERROR');

            if ($isEventLogEnabled) {
                Logger::Add($errorMessage, __METHOD__ . '();');
            }
            $result->addError(new Error($errorMessage));
            return $result;
        }

        try {
            $orderProps = $order->getPropertyCollection();
            if ($orderProps === null) {
                $errorMessage = Loc::getMessage('GET_ORDER_PROPS_IN_CALCULATE_ERROR');
                if ($isEventLogEnabled) {
                    Logger::Add($errorMessage, __METHOD__ . '();');
                }
                $result->addError(new Error($errorMessage));
                return $result;
            }

        } catch (\Exception $e) {
            if ($isEventLogEnabled) {
                Logger::Add($e->getMessage(), __METHOD__ . '();', $e);
            }
            $result->addError(new Error($e->getMessage()));
            return $result;
        }

        if (!$locationProp = $orderProps->getItemByOrderPropertyCode($options->getLocation())) {
            $errorMessage = Loc::getMessage('LOCATION_PROP_IS_NOT_SET_OR_EMPTY');

            if ($isEventLogEnabled) {
                Logger::Add($errorMessage, __METHOD__ . '();');
            }
            $result->addError(new Error($errorMessage));
            return $result;
        }

        if (!$locationCode = $locationProp->getValue()){
            $errorMessage = Loc::getMessage('CAN_NOT_GET_LOCATION_CODE');

            if ($isEventLogEnabled) {
                Logger::Add($errorMessage, __METHOD__ . '();');
            }
            $result->addError(new Error($errorMessage));
            return $result;
        }

        try {
            $city = CitiesFullTable::getBbCityByBitrixLocationCode($locationCode);
        } catch (\Exception $e) {
            if ($isEventLogEnabled) {
                Logger::Add($e->getMessage(), __METHOD__ . '();', $e);
            }

            $result->addError(new Error($e->getMessage()));
            return $result;
        }

        if ($options->isPickup() && !$city->getPickupPoint()) {
            $errorMessage = Loc::getMessage('NO_PICKUP_POINT_IN_SELECTED_CITY', ['#CITY#' => $city->getName(), '#CODE#' => $city->getCode()]);
            if ($isEventLogEnabled) {
                Logger::Add($errorMessage, __METHOD__ . '();');
            }

            $result->addError(new Error($errorMessage));
            return $result;
        }

        if ($options->isCourier() && !$city->getCourierDelivery()) {
            $errorMessage = Loc::getMessage('NO_COURIER_DELIVERY_IN_SELECTED_CITY', ['#CITY#' => $city->getName(), '#CODE#' => $city->getCode()]);
            if ($isEventLogEnabled) {
                Logger::Add($errorMessage, __METHOD__ . '();');
            }

            $result->addError(new Error($errorMessage));
            return $result;
        }

        if (!$options->isTestMode()) {
            try {
                $widgetSettings = $api->widgetSettings();
            } catch (\Exception $e) {
                if ($isEventLogEnabled) {
                    Logger::Add($e->getMessage(), __METHOD__ . '();', $e);
                }

                $result->addError(new Error($e->getMessage()));
                return $result;
            }

            if ($widgetSettings->isCityDisabled($city->getCode())) {
                $errorMessage = Loc::getMessage('DELIVERY_IN_SELECTED_CITY_IS_DISABLED_IN_LK');
                if ($isEventLogEnabled) {
                    Logger::Add($errorMessage, __METHOD__ . '();');
                }
                $result->addError(new Error($errorMessage));
                return $result;
            }
        } else {
            $widgetSettings = null;
        }

        $orderSum = $basket->getPrice();
        $paySum = self::getPaySum($orderSum, $options, $order->getPaymentCollection());
        $useShopSettings =  $options->getUseShopSettings() === '1' ? '1' : '0';

        if (!$weights = BoxSizesCalculator::getBoxSizesArrayFromOrderItems($basket->toArray(), $deliveryId)) {
            $errorMessage = Loc::getMessage('CAN_NOT_GET_DIMENSIONS');
            if ($isEventLogEnabled) {
                Logger::Add($errorMessage, __METHOD__ . '();');
            }
            $result->addError(new Error($errorMessage));
            return $result;
        }

        if ($options->isCalcSourceProfile()) {
            $weights[0]['Weight'] += (int)$options->getWeight();

            if ($weights[0]['Weight'] >= CountriesSettings::LOAD_LIMIT) {
                $errorMessage = Loc::getMessage('WEIGHT_LIMIT_EXCEEDED');
                if ($isEventLogEnabled) {
                    Logger::Add($errorMessage, __METHOD__ . '();', $deliveryId);
                }
                $result->addError(new Error($errorMessage));
                return $result;
            }
        }

        $deliveryCalculationParams = new DeliveryCalculationParams();

        if ($externalData !== null && $externalData->isSuccess()) {
            $externalData = $externalData->getData();

            if ($targetStart = PointsForParcelsTable::getPointCodeByName($externalData['targetStart'])) {
                $deliveryCalculationParams->setTargetStart($targetStart);
            } else {
                $errorMessage = Loc::getMessage('CAN_NOT_GET_POINT_FOR_PARCEL_CODE');
                if ($isEventLogEnabled) {
                    Logger::Add($errorMessage, __METHOD__ . '();');
                }
                $result->addError(new Error($errorMessage));
                return $result;
            }

            isset($externalData['boxSizes']) ? $deliveryCalculationParams->setBoxSizes($externalData['boxSizes']) : $deliveryCalculationParams->setBoxSizes($weights);
            $deliveryCalculationParams->setOrderSum($externalData['orderSum']);
            $deliveryCalculationParams->setPaySum($externalData['paySum']);
        } else {
            $deliveryCalculationParams->setTargetStart($options->getPointForParcelCode());
            $deliveryCalculationParams->setBoxSizes($weights);
            $deliveryCalculationParams->setOrderSum($orderSum);
            $deliveryCalculationParams->setPaySum($paySum);
        }

        $deliveryCalculationParams->setDeliveryType($options->getProfileType());
        $deliveryCalculationParams->setRecipientCityId($city->getCode());
        $deliveryCalculationParams->setUseShopSettings($useShopSettings);
        $deliveryCalculationParams->setCmsName(Options::getCmsName());
        $deliveryCalculationParams->setVersion(Options::getModuleVersion());
        $deliveryCalculationParams->setUrl($_SERVER['HTTP_HOST']);

        if ($options->isCourier() && $options->getZipCheck() && $city->getCountryCode() === CitiesFullTable::RU_COUNTRY_CODE) {
            $zip = self::getPropValue($orderProps->getItemByOrderPropertyCode($options->getZip()));

            if (!$zip) {
                $errorMessage = Loc::getMessage('ZIP_PROP_IS_NOT_SET_OR_EMPTY');
                if ($isEventLogEnabled) {
                    Logger::Add($errorMessage, __METHOD__ . '();');
                }
            } else {
                try {
                    $zipCheck = $api->zipCheck($zip);
                } catch (BoxberryApiException $e) {
                    if ($isEventLogEnabled) {
                        Logger::Add($e->getMessage(), __METHOD__ . '();', $e);
                    }
                    $result->addError(new Error($e->getMessage()));
                    return $result;
                }

                if ($zipCheck->isExpressDelivery()) {
                    $deliveryCalculationParams->setZip($zip);
                }
            }
        }

        try {
            $cost = $api->deliveryCalculation($deliveryCalculationParams);
        } catch (BoxberryApiException $e) {
            if ($isEventLogEnabled) {
                Logger::Add($e->getMessage(), __METHOD__ . '();', $e);
            }
            $result->addError(new Error($e->getMessage()));

            return $result;
        }

        static $descriptionAdded = [];

        if ($cost->getPriceBasePickup() && $options->isPickup()) {

            if ($checkCompatible) {
                return $result;
            }

            $result = self::correctPriceWithSettings($cost->getTotalPricePickup(), $paySum, $orderSum, $options, $result);
            $result->setPeriodDescription(self::correctDeliveryDaysWithSettings($cost->getDeliveryPeriodPickup(), $options, $widgetSettings));

            if (BoxberryOrderHandler::isAdminSection()) {
                return $result;
            }

            $boxWeights = BoxSizesCalculator::getAllBoxesWeight($weights);

            $widget = new Widget();
            $widget->setDivId($widget->getName() . '_' . $deliveryId);
            $widget->setWidgetKey($options->isTestMode() ? Client::TEST_WIDGET_KEY : $options->getWidgetKey());
            self::setWidgetCustomCity($widget, $city);
            $widget->setOrderSum($orderSum);
            $widget->setWeight($boxWeights);
            $widget->setPaySum($paySum);
            $widget->setHeight(0);
            $widget->setWidth(0);
            $widget->setDepth(0);
            $widget->setIsAddressOneProp($options->isAddressOneProp());
            $widget->setPvzAddress($widget->isPvzAddressSet() ? $widget->getPvzAddress() : '');
            $widget->setPvzId($widget->isPvzIdSet() ? $widget->getPvzId() : '');
            $widget->setPvzProfileIds(self::getPvzProfilesIds());
            $widget->setConsoleLogEnabled(Options::isConsoleLogEnabled());
            $widget->setAddressPvzReadOnly($options->isAddressPvzReadOnly());

            self::setAddressPropIdToWidget($widget, $options, $orderProps);

            if ($widget->getCityCode() !== $city->getCode()) {
                $widget->removePvzAndAddress();
            }

            $widget->setCityCode($city->getCode());
            $widget->setAddToDescription('');
            $widget->setAddAfterDescription('');
            $widget->setAddToCustomId('');
            $widget->setCustomId('');

            if ($options->isAddLinkDefault()) {
                $result->setDescription($widget->getDivElement());
            } elseif ($options->isAddLinkAfterDescription()) {
                $widget->setAddAfterDescription($widget->getDivElement());
            } elseif ($options->isAddLinkToDescription()) {
                $widget->setAddToDescription($widget->getButtonHtml());

                if (!in_array($deliveryId, $descriptionAdded)) {
                    $shipment->getDelivery()->setDescription('<div id="' . $widget->getDivId() . '"></div>');
                    $descriptionAdded[] = $deliveryId;
                }

            } elseif ($options->isAddLinkToPeriod()) {
                $result->setPeriodDescription($result->getPeriodDescription() . $widget->getDivElement());
            } elseif ($options->isAddLinkToCustom()) {
                $widget->setAddToCustomId($widget->getDivElement());
                $widget->setCustomId($options->getCustomElementId());
            }

            return $result;
        }

        if ($cost->getPriceBaseCourier() && $options->isCourier()) {

            if ($checkCompatible) {
                return $result;
            }

            $result = self::correctPriceWithSettings($cost->getTotalPriceCourier(), $paySum, $orderSum, $options, $result);
            $result->setPeriodDescription(self::correctDeliveryDaysWithSettings($cost->getDeliveryPeriodCourier(), $options, $widgetSettings));
            return $result;
        }

        $errorMessage = Loc::getMessage('DELIVERY_IN_SELECTED_CITY_IS_NOT_AVAILABLE', ['#LOCATION_CODE#' => $locationCode]);
        if ($isEventLogEnabled) {
            Logger::Add($errorMessage, __METHOD__ . '();');
        }

        $result->addError(new Error($errorMessage));

        return $result;

    }


}