<?php

namespace Boxberry\Bitrix\Services;

use Bitrix\Main\Application;
use Bitrix\Main\Error;
use Bitrix\Main\IO\Directory;
use Bitrix\Main\IO\File;
use Bitrix\Main\Result;
use Bitrix\Main\Localization\Loc;
use Boxberry\Api\Client;
use Boxberry\Bitrix\Helpers\Options;
use CBXArchive;
use CUpdateSystem;

class Updater
{
    public static function execute(string $url, string $version, string $archiveFormat): Result
    {
        $result = new Result();

        if (!$updateArchivePath = Client::getModuleUpdate($url, $version, $archiveFormat)) {
            $result->addError(new Error(Loc::getMessage('CAN_NOT_DOWNLOAD_ARCHIVE_BY_LINK') . $url));
            return $result;
        }

        if (!File::isFileExists($updateArchivePath)) {
            $result->addError(new Error(Loc::getMessage('CAN_NOT_FIND_ARCHIVE_WITH_UPDATE')));
            return $result;
        }

        $updatePath = Application::getDocumentRoot() . '/bitrix/updates/' . $version . '/' . Options::getModuleId() . '/';

        if (!Directory::isDirectoryExists($updatePath)) {
            Directory::createDirectory($updatePath);
        }

        $makeWritable = new Directory($updatePath);

        try {
            $makeWritable->markWritable();
        } catch (\Exception $e) {
            $result->addError(new Error(Loc::getMessage('ERROR') . $e->getMessage()));
            return $result;
        }

        $unpackResult = self::unpackGzipArchive($updateArchivePath, $updatePath);

        if (!$unpackResult->isSuccess()) {
            return $unpackResult;
        }

        $runUpdateKernelResult = self::runUpdateKernel($version);

        if (!$runUpdateKernelResult->isSuccess()) {
            return $runUpdateKernelResult;
        }

        $updateUploadPath = Application::getDocumentRoot() . Client::FILE_UPLOAD_DIR . 'updates';

        if (Directory::isDirectoryExists($updateUploadPath)) {
            Directory::deleteDirectory($updateUploadPath);
        }

        Client::logSuccessUpdate();

        $result->setData(['message' => Loc::getMessage('MODULE_SUCCESSFULLY_UPDATED_TO_VERSION') .  $version, 'success' => true, 'version' => $version]);
        return $result;

    }

    private static function unpackGzipArchive($archivePath, $destinationPath): Result
    {
        global $USER;
        $result = new Result();
        $arc = CBXArchive::GetArchive($archivePath);
        $arc->SetOptions([
            'REMOVE_PATH' => $destinationPath,
            'UNPACK_REPLACE' => true,
            'CHECK_PERMISSIONS' => !$USER->IsAdmin()
        ]);
        $unPackResult = $arc->Unpack($destinationPath);

        if (!$unPackResult) {
            $result->addErrors($arc->GetErrors());
        }

        return $result;
    }

    private static function runUpdateKernel($version): Result
    {
        require_once(Application::getDocumentRoot() . '/bitrix/modules/main/classes/general/update_class.php');
        Loc::loadMessages(Application::getDocumentRoot() . '/bitrix/modules/main/classes/general/update_client.php');

        $result = new Result();
        $updater = new CUpdateSystem();

        $updatesDir = $version;
        $arModules = [Options::getModuleId()];
        $strErrorMessage = '';
        $arErrorModules = [];
        $arSuccessModules = [];

        if (!$updater->UpdateKernel($updatesDir, $arModules, $strErrorMessage, $arErrorModules, $arSuccessModules)) {
            $result->addErrors($arErrorModules);
        }

        return $result;
    }

}